/***************************************************************************//**
* \file CapSenseButton_INT.c
* \version 7.0
*
* \brief
*   This file contains the source code for implementation of the Component's
*   Interrupt Service Routine (ISR).
*
* \see CapSenseButton v7.0 Datasheet
*
*//*****************************************************************************
* Copyright (2016-2019), Cypress Semiconductor Corporation.
********************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress) and is
* protected by and subject to worldwide patent protection (United States and
* foreign), United States copyright laws and international treaty provisions.
* Cypress hereby grants to licensee a personal, non-exclusive, non-transferable
* license to copy, use, modify, create derivative works of, and compile the
* Cypress Source Code and derivative works for the sole purpose of creating
* custom software in support of licensee product to be used only in conjunction
* with a Cypress integrated circuit as specified in the applicable agreement.
* Any reproduction, modification, translation, compilation, or representation of
* this software except as specified above is prohibited without the express
* written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the
* materials described herein. Cypress does not assume any liability arising out
* of the application or use of any product or circuit described herein. Cypress
* does not authorize its products for use as critical components in life-support
* systems where a malfunction or failure may reasonably be expected to result in
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of
* such use and in doing so indemnifies Cypress against all charges. Use may be
* limited by and subject to the applicable Cypress software license agreement.
*******************************************************************************/
#include "cytypes.h"
#include "cyfitter.h"
#include "CapSenseButton_Configuration.h"
#include "CapSenseButton_Structure.h"
#include "CapSenseButton_Sensing.h"
#if (CapSenseButton_ENABLE == CapSenseButton_CSD_EN)
    #include "CapSenseButton_SensingCSD_LL.h"
#endif /* (CapSenseButton_ENABLE == CapSenseButton_CSD_EN) */
#include "cyapicallbacks.h"

/*******************************************************************************
* Static Function Prototypes
*******************************************************************************/
/**
* \cond SECTION_C_INTERNAL
* \addtogroup group_c_internal
* \{
*/

#if (((CapSenseButton_ENABLE == CapSenseButton_CSD_EN) || (CapSenseButton_ENABLE == CapSenseButton_CSD_CSX_EN)) && \
     (CapSenseButton_ENABLE == CapSenseButton_MULTI_FREQ_SCAN_EN))
    static void CapSenseButton_SsNextFrequencyScan(void);
#endif /* (((CapSenseButton_ENABLE == CapSenseButton_CSD_EN) || (CapSenseButton_ENABLE == CapSenseButton_CSD_CSX_EN)) && \
            (CapSenseButton_ENABLE == CapSenseButton_MULTI_FREQ_SCAN_EN)) */

#if ((CapSenseButton_ENABLE == CapSenseButton_CSD_EN) || (CapSenseButton_ENABLE == CapSenseButton_CSD_CSX_EN))
    static void CapSenseButton_SsCSDPostScan(void);
    static void CapSenseButton_SsCSDInitNextScan(void);
#endif /* ((CapSenseButton_ENABLE == CapSenseButton_CSD_EN) || (CapSenseButton_ENABLE == CapSenseButton_CSD_CSX_EN)) */
/** \}
* \endcond */


/**
* \cond SECTION_C_INTERRUPT
* \addtogroup group_c_interrupt
* \{
*/


#if ((CapSenseButton_ENABLE == CapSenseButton_CSD_EN) || (CapSenseButton_ENABLE == CapSenseButton_CSD_CSX_EN))

#if (CapSenseButton_ENABLE == CapSenseButton_CSDV2)
    /* Fourth-generation HW block part */

    /*******************************************************************************
    * Function Name: CapSenseButton_CSDPostSingleScan
    ****************************************************************************//**
    *
    * \brief
    *  This is an internal ISR function for the single-sensor scanning implementation.
    *
    * \details
    *  This ISR handler is triggered when the user calls the
    *  CapSenseButton_CSDScanExt() function.
    *
    *  The following tasks are performed for Third-generation HW block:
    *    1. Disable the CSD interrupt.
    *    2. Read the Counter register and update the data structure with raw data.
    *    3. Connect the Vref buffer to the AMUX bus.
    *    4. Update the Scan Counter.
    *    5. Reset the BUSY flag.
    *    6. Enable the CSD interrupt.
    *
    *  The following tasks are performed for Fourth-generation HW block:
    *    1. Check if the raw data is not noisy.
    *    2. Read the Counter register and update the data structure with raw data.
    *    3. Configure and start the scan for the next frequency if the
    *      multi-frequency is enabled.
    *    4. Update the Scan Counter.
    *    5. Reset the BUSY flag.
    *    6. Enable the CSD interrupt.
    *
    *  The ISR handler changes the IMO and initializes scanning for the next frequency
    *  channels when multi-frequency scanning is enabled.
    *
    *  This function has two Macro Callbacks that allow calling the user code
    *  from macros specified in Component's generated code. Refer to the
    *  \ref group_c_macrocallbacks section of the PSoC Creator User Guide
    *  for details.
    *
    *******************************************************************************/
    CY_ISR(CapSenseButton_CSDPostSingleScan)
    {
        #ifdef CapSenseButton_ENTRY_CALLBACK
            CapSenseButton_EntryCallback();
        #endif /* CapSenseButton_ENTRY_CALLBACK */

        /* Clear pending interrupt */
        CY_SET_REG32(CapSenseButton_INTR_PTR, CapSenseButton_INTR_SET_MASK);

        #if (CapSenseButton_ENABLE == CapSenseButton_CSD_NOISE_METRIC_EN)
            if ((CapSenseButton_CSD_NOISE_METRIC_TH < ((CY_GET_REG32(CapSenseButton_RESULT_VAL1_PTR) &
                                                        CapSenseButton_RESULT_VAL1_BAD_CONVS_MASK) >>
                                                        CapSenseButton_RESULT_VAL1_BAD_CONVS_SHIFT)) &&
                                                        (0u < CapSenseButton_badConversionsNum))
            {
                /* Decrement bad conversions number */
                CapSenseButton_badConversionsNum--;

                /* Start the re-scan */
                CY_SET_REG32(CapSenseButton_SEQ_START_PTR, CapSenseButton_SEQ_START_AZ0_SKIP_MASK |
                                                             CapSenseButton_SEQ_START_AZ1_SKIP_MASK |
                                                             CapSenseButton_SEQ_START_START_MASK);
            }
            else
            {
        #endif /* (CapSenseButton_ENABLE == CapSenseButton_CSD_NOISE_METRIC_EN) */

            CapSenseButton_SsCSDPostScan();

            #if (CapSenseButton_ENABLE == CapSenseButton_MULTI_FREQ_SCAN_EN)
                if (CapSenseButton_FREQ_CHANNEL_2 > CapSenseButton_scanFreqIndex)
                {
                    /* Scan the next channel */
                    CapSenseButton_SsNextFrequencyScan();
                }
                else
                {
                    /* All channels are scanned. Set IMO to zero channel */
                    CapSenseButton_scanFreqIndex = CapSenseButton_FREQ_CHANNEL_0;

                    #if (CapSenseButton_MFS_IMO == CapSenseButton_MFS_METHOD)
                        CapSenseButton_SsChangeImoFreq(CapSenseButton_FREQ_CHANNEL_0);
                    #else
                        CapSenseButton_SsChangeClkFreq(CapSenseButton_FREQ_CHANNEL_0);
                    #endif /* (CapSenseButton_MFS_IMO == CapSenseButton_MFS_METHOD) */

                    #if (CapSenseButton_ENABLE == CapSenseButton_BLOCK_OFF_AFTER_SCAN_EN)
                        /* Disable Fourth-generation HW block */
                        CY_SET_REG32(CapSenseButton_CONFIG_PTR, CapSenseButton_configCsd);
                    #endif /* (CapSenseButton_ENABLE == CapSenseButton_BLOCK_OFF_AFTER_SCAN_EN) */

                    /* Update Scan Counter */
                    CapSenseButton_dsRam.scanCounter++;

                    /* Sensor is totally scanned. Reset BUSY flag */
                    CapSenseButton_dsRam.status &= ~(CapSenseButton_SW_STS_BUSY | CapSenseButton_WDGT_SW_STS_BUSY);
                }
            #else
                {
                    #if (CapSenseButton_ENABLE == CapSenseButton_BLOCK_OFF_AFTER_SCAN_EN)
                        /* Disable Fourth-generation HW block */
                        CY_SET_REG32(CapSenseButton_CONFIG_PTR, CapSenseButton_configCsd);
                    #endif /* (CapSenseButton_ENABLE == CapSenseButton_BLOCK_OFF_AFTER_SCAN_EN) */

                    /* Update Scan Counter */
                    CapSenseButton_dsRam.scanCounter++;

                    /* Sensor is totally scanned. Reset BUSY flag */
                    CapSenseButton_dsRam.status &= ~(CapSenseButton_SW_STS_BUSY | CapSenseButton_WDGT_SW_STS_BUSY);
                }
            #endif /* (CapSenseButton_ENABLE == CapSenseButton_MULTI_FREQ_SCAN_EN) */
    #if (CapSenseButton_ENABLE == CapSenseButton_CSD_NOISE_METRIC_EN)
        }
    #endif /* (CapSenseButton_ENABLE == CapSenseButton_CSD_NOISE_METRIC_EN) */

        #ifdef CapSenseButton_EXIT_CALLBACK
            CapSenseButton_ExitCallback();
        #endif /* CapSenseButton_EXIT_CALLBACK */
    }


    /*******************************************************************************
    * Function Name: CapSenseButton_CSDPostMultiScan
    ****************************************************************************//**
    *
    * \brief
    *  This is an internal ISR function for the multiple-sensor scanning
    *  implementation.
    *
    * \details
    *  This ISR handler is triggered when the user calls the
    *  CapSenseButton_Scan() or CapSenseButton_ScanAllWidgets() APIs.
    *
    *  The following tasks are performed:
    *    1. Disable the CSD interrupt.
    *    2. Read the Counter register and update the data structure with raw data.
    *    3. Connect the Vref buffer to the AMUX bus.
    *    4. Disable the CSD block (after the widget has been scanned).
    *    5. Update the Scan Counter.
    *    6. Reset the BUSY flag.
    *    7. Enable the CSD interrupt.
    *
    *  The ISR handler initializes scanning for the previous sensor when the
    *  widget has more than one sensor.
    *  The ISR handler initializes scanning for the next widget when the
    *  CapSenseButton_ScanAllWidgets() APIs are called and the project has
    *  more than one widget.
    *  The ISR handler changes the IMO and initializes scanning for the next
    *  frequency channels when multi-frequency scanning is enabled.
    *
    *  This function has two Macro Callbacks that allow calling the user
    *  code from macros specified in Component's generated code. Refer to the
    *  \ref group_c_macrocallbacks section of the PSoC Creator User Guide
    *  for details.
    *
    *******************************************************************************/
    CY_ISR(CapSenseButton_CSDPostMultiScan)
    {
        /* Declare and initialize ptr to sensor IO structure to appropriate address */
        CapSenseButton_FLASH_IO_STRUCT const *curSnsIOPtr = (CapSenseButton_FLASH_IO_STRUCT const *)
                                                          CapSenseButton_dsFlash.wdgtArray[CapSenseButton_widgetIndex].ptr2SnsFlash
                                                          + CapSenseButton_sensorIndex;

        #ifdef CapSenseButton_ENTRY_CALLBACK
            CapSenseButton_EntryCallback();
        #endif /* CapSenseButton_ENTRY_CALLBACK */

        /* Clear pending interrupt */
        CY_SET_REG32(CapSenseButton_INTR_PTR, CapSenseButton_INTR_SET_MASK);

        #if (CapSenseButton_ENABLE == CapSenseButton_CSD_NOISE_METRIC_EN)
            if ((CapSenseButton_CSD_NOISE_METRIC_TH < ((CY_GET_REG32(CapSenseButton_RESULT_VAL1_PTR) &
                                                      CapSenseButton_RESULT_VAL1_BAD_CONVS_MASK) >>
                                                      CapSenseButton_RESULT_VAL1_BAD_CONVS_SHIFT)) &&
                                                      (0u < CapSenseButton_badConversionsNum))
            {
                /* Decrement bad conversions number */
                CapSenseButton_badConversionsNum--;

                /* Start the re-scan */
                CY_SET_REG32(CapSenseButton_SEQ_START_PTR, CapSenseButton_SEQ_START_AZ0_SKIP_MASK |
                                                             CapSenseButton_SEQ_START_AZ1_SKIP_MASK |
                                                             CapSenseButton_SEQ_START_START_MASK);
            }
            else
            {
        #endif /* (CapSenseButton_ENABLE == CapSenseButton_CSD_NOISE_METRIC_EN) */

            CapSenseButton_SsCSDPostScan();

            /* Disable sensor when all frequency channels are scanned */
            #if (CapSenseButton_ENABLE == CapSenseButton_MULTI_FREQ_SCAN_EN)
                if (CapSenseButton_FREQ_CHANNEL_2 == CapSenseButton_scanFreqIndex)
            #endif /* (CapSenseButton_ENABLE == CapSenseButton_MULTI_FREQ_SCAN_EN) */
            {
                /* Disable sensor */
                CapSenseButton_CSDDisconnectSns(curSnsIOPtr);
            }

            #if (CapSenseButton_ENABLE == CapSenseButton_MULTI_FREQ_SCAN_EN)
                if (CapSenseButton_FREQ_CHANNEL_2 > CapSenseButton_scanFreqIndex)
                {
                     /* Scan the next channel */
                    CapSenseButton_SsNextFrequencyScan();
                }
                else
                {
                     /* All channels are scanned. Set IMO to zero channel */
                    CapSenseButton_scanFreqIndex = CapSenseButton_FREQ_CHANNEL_0;

                    #if (CapSenseButton_MFS_IMO == CapSenseButton_MFS_METHOD)
                        CapSenseButton_SsChangeImoFreq(CapSenseButton_FREQ_CHANNEL_0);
                    #else
                        CapSenseButton_SsChangeClkFreq(CapSenseButton_FREQ_CHANNEL_0);
                    #endif /* (CapSenseButton_MFS_IMO == CapSenseButton_MFS_METHOD) */

                     /* Scan the next sensor */
                    CapSenseButton_SsCSDInitNextScan();
                }
            #else
                /* Scan the next sensor */
                CapSenseButton_SsCSDInitNextScan();
            #endif /* (CapSenseButton_ENABLE == CapSenseButton_MULTI_FREQ_SCAN_EN) */
        #if (CapSenseButton_ENABLE == CapSenseButton_CSD_NOISE_METRIC_EN)
            }
        #endif /* (CapSenseButton_ENABLE == CapSenseButton_CSD_NOISE_METRIC_EN) */

        #ifdef CapSenseButton_EXIT_CALLBACK
            CapSenseButton_ExitCallback();
        #endif /* CapSenseButton_EXIT_CALLBACK */
    }


    #if (CapSenseButton_ENABLE == CapSenseButton_CSD_GANGED_SNS_EN)
    /*******************************************************************************
    * Function Name: CapSenseButton_CSDPostMultiScanGanged
    ****************************************************************************//**
    *
    * \brief
    *  This is an internal ISR function for the multiple-sensor scanning
    *  implementation for ganged sensors.
    *
    * \details
    *  This ISR handler is triggered when the user calls the
    *  CapSenseButton_Scan() API for a ganged sensor or the
    *  CapSenseButton_ScanAllWidgets() API in the project with ganged sensors.
    *
    *  The following tasks are performed:
    *    1. Disable the CSD interrupt.
    *    2. Read the Counter register and update the data structure with raw data.
    *    3. Connect the Vref buffer to the AMUX bus.
    *    4. Disable the CSD block (after the widget has been scanned).
    *    5. Update the Scan Counter.
    *    6. Reset the BUSY flag.
    *    7. Enable the CSD interrupt.
    *
    *  The ISR handler initializes scanning for the previous sensor when the
    *  widget has more than one sensor.
    *  The ISR handler initializes scanning for the next widget when the
    *  CapSenseButton_ScanAllWidgets() APIs are called and the project has
    *  more than one widget.
    *  The ISR handler changes the IMO and initializes scanning for the next
    *  frequency channels when multi-frequency scanning is enabled.
    *
    *  This function has two Macro Callbacks that allow calling the user
    *  code from macros specified in Component's generated code. Refer to the
    *  \ref group_c_macrocallbacks section of the PSoC Creator User Guide
    *  for details.
    *
    *******************************************************************************/
    CY_ISR(CapSenseButton_CSDPostMultiScanGanged)
    {
        #ifdef CapSenseButton_ENTRY_CALLBACK
            CapSenseButton_EntryCallback();
        #endif /* CapSenseButton_ENTRY_CALLBACK */

        /* Clear pending interrupt */
        CY_SET_REG32(CapSenseButton_INTR_PTR, CapSenseButton_INTR_SET_MASK);

        #if (CapSenseButton_ENABLE == CapSenseButton_CSD_NOISE_METRIC_EN)
            if ((CapSenseButton_CSD_NOISE_METRIC_TH < ((CY_GET_REG32(CapSenseButton_RESULT_VAL1_PTR) &
                                                      CapSenseButton_RESULT_VAL1_BAD_CONVS_MASK) >>
                                                      CapSenseButton_RESULT_VAL1_BAD_CONVS_SHIFT)) &&
                                                      (0u < CapSenseButton_badConversionsNum))
            {
                /* Decrement bad conversions number */
                CapSenseButton_badConversionsNum--;

                /* Start the re-scan */
                CY_SET_REG32(CapSenseButton_SEQ_START_PTR, CapSenseButton_SEQ_START_AZ0_SKIP_MASK |
                                                             CapSenseButton_SEQ_START_AZ1_SKIP_MASK |
                                                             CapSenseButton_SEQ_START_START_MASK);
            }
            else
            {
        #endif /* (CapSenseButton_ENABLE == CapSenseButton_CSD_NOISE_METRIC_EN) */

            CapSenseButton_SsCSDPostScan();

            #if (CapSenseButton_ENABLE == CapSenseButton_MULTI_FREQ_SCAN_EN)
                if (CapSenseButton_FREQ_CHANNEL_2 == CapSenseButton_scanFreqIndex)
            #endif /* (CapSenseButton_ENABLE == CapSenseButton_MULTI_FREQ_SCAN_EN) */
            {
                CapSenseButton_SsCSDDisconnectSnsExt((uint32)CapSenseButton_widgetIndex, (uint32)CapSenseButton_sensorIndex);
            }

            #if (CapSenseButton_ENABLE == CapSenseButton_MULTI_FREQ_SCAN_EN)
                if (CapSenseButton_FREQ_CHANNEL_2 > CapSenseButton_scanFreqIndex)
                {
                     /* Scan the next channel */
                    CapSenseButton_SsNextFrequencyScan();
                }
                else
                {
                    /* All channels are scanned. Set IMO to zero channel */
                    CapSenseButton_scanFreqIndex = CapSenseButton_FREQ_CHANNEL_0;

                    #if (CapSenseButton_MFS_IMO == CapSenseButton_MFS_METHOD)
                        CapSenseButton_SsChangeImoFreq(CapSenseButton_FREQ_CHANNEL_0);
                    #else
                        CapSenseButton_SsChangeClkFreq(CapSenseButton_FREQ_CHANNEL_0);
                    #endif /* (CapSenseButton_MFS_IMO == CapSenseButton_MFS_METHOD) */

                     /* Scan the next sensor */
                    CapSenseButton_SsCSDInitNextScan();
                }
            #else
                 /* Scan the next sensor */
                CapSenseButton_SsCSDInitNextScan();
            #endif /* (CapSenseButton_ENABLE == CapSenseButton_MULTI_FREQ_SCAN_EN) */
        #if (CapSenseButton_ENABLE == CapSenseButton_CSD_NOISE_METRIC_EN)
            }
        #endif /* (CapSenseButton_ENABLE == CapSenseButton_CSD_NOISE_METRIC_EN) */

        #ifdef CapSenseButton_EXIT_CALLBACK
            CapSenseButton_ExitCallback();
        #endif /* CapSenseButton_EXIT_CALLBACK */
    }
    #endif /* (CapSenseButton_ENABLE == CapSenseButton_CSD_GANGED_SNS_EN) */

#else

    /* Third-generation HW block part */

    /*******************************************************************************
    * Function Name: CapSenseButton_CSDPostSingleScan
    ****************************************************************************//**
    *
    * \brief
    *  This is an internal ISR function for the single-sensor scanning implementation.
    *
    * \details
    *  This ISR handler is triggered when the user calls the
    *  CapSenseButton_CSDScanExt() function.
    *
    *  The following tasks are performed for Third-generation HW block:
    *    1. Disable the CSD interrupt.
    *    2. Read the Counter register and update the data structure with raw data.
    *    3. Connect the Vref buffer to the AMUX bus.
    *    4. Update the Scan Counter.
    *    5. Reset the BUSY flag.
    *    6. Enable the CSD interrupt.
    *
    *  The following tasks are performed for Fourth-generation HW block:
    *    1. Check if the raw data is not noisy.
    *    2. Read the Counter register and update the data structure with raw data.
    *    3. Configure and start the scan for the next frequency if the
    *      multi-frequency is enabled.
    *    4. Update the Scan Counter.
    *    5. Reset the BUSY flag.
    *    6. Enable the CSD interrupt.
    *
    *  The ISR handler changes the IMO and initializes scanning for the next frequency
    *  channels when multi-frequency scanning is enabled.
    *
    *  This function has two Macro Callbacks that allow calling the user code
    *  from macros specified in Component's generated code. Refer to the
    *  \ref group_c_macrocallbacks section of the PSoC Creator User Guide
    *  for details.
    *
    *******************************************************************************/
    CY_ISR(CapSenseButton_CSDPostSingleScan)
    {
        #ifdef CapSenseButton_ENTRY_CALLBACK
            CapSenseButton_EntryCallback();
        #endif /* CapSenseButton_ENTRY_CALLBACK */

        /* Clear pending interrupt */
        CY_SET_REG32(CapSenseButton_INTR_PTR, CapSenseButton_INTR_SET_MASK);

        /* Read Rawdata */
        CapSenseButton_SsCSDPostScan();

        #if (CapSenseButton_ENABLE == CapSenseButton_MULTI_FREQ_SCAN_EN)
            if (CapSenseButton_FREQ_CHANNEL_2 > CapSenseButton_scanFreqIndex)
            {
                /* Connect Vref Buffer to AMUX bus. Third-generation HW block is enabled */
                CY_SET_REG32(CapSenseButton_CONFIG_PTR, CapSenseButton_configCsd | CapSenseButton_CTANK_PRECHARGE_CONFIG_CSD_EN);

                CapSenseButton_SsNextFrequencyScan();
            }
            else
            {
                CapSenseButton_scanFreqIndex = CapSenseButton_FREQ_CHANNEL_0;

                #if (CapSenseButton_MFS_IMO == CapSenseButton_MFS_METHOD)
                    CapSenseButton_SsChangeImoFreq(CapSenseButton_FREQ_CHANNEL_0);
                #else
                    CapSenseButton_SsChangeClkFreq(CapSenseButton_FREQ_CHANNEL_0);
                #endif /* (CapSenseButton_MFS_IMO == CapSenseButton_MFS_METHOD) */

                #if (CapSenseButton_ENABLE == CapSenseButton_BLOCK_OFF_AFTER_SCAN_EN)
                    /* Disable Third-generation HW block. Connect Vref Buffer to AMUX bus */
                    #if ((CapSenseButton_CSH_PRECHARGE_IO_BUF == CapSenseButton_CSD_CSH_PRECHARGE_SRC) &&\
                         (CapSenseButton_ENABLE == CapSenseButton_CSD_SHIELD_TANK_EN))
                        CY_SET_REG32(CapSenseButton_CONFIG_PTR, CapSenseButton_configCsd | CapSenseButton_CMOD_PRECHARGE_CONFIG);
                    #else
                        CY_SET_REG32(CapSenseButton_CONFIG_PTR, CapSenseButton_configCsd | CapSenseButton_CTANK_PRECHARGE_CONFIG);
                    #endif
                #else
                    /* Connect Vref Buffer to AMUX bus. Third-generation HW block is enabled */
                    CY_SET_REG32(CapSenseButton_CONFIG_PTR, CapSenseButton_configCsd | CapSenseButton_CTANK_PRECHARGE_CONFIG_CSD_EN);
                #endif

                /* Update Scan Counter */
                CapSenseButton_dsRam.scanCounter++;

                /* Sensor is totally scanned. Reset BUSY flag */
                CapSenseButton_dsRam.status &= ~(CapSenseButton_SW_STS_BUSY | CapSenseButton_WDGT_SW_STS_BUSY);
            }
        #else
            {
                #if (CapSenseButton_ENABLE == CapSenseButton_BLOCK_OFF_AFTER_SCAN_EN)
                    /* Disable Third-generation HW block. Connect Vref Buffer to AMUX bus */
                    #if ((CapSenseButton_CSH_PRECHARGE_IO_BUF == CapSenseButton_CSD_CSH_PRECHARGE_SRC) &&\
                         (CapSenseButton_ENABLE == CapSenseButton_CSD_SHIELD_TANK_EN))
                        CY_SET_REG32(CapSenseButton_CONFIG_PTR, CapSenseButton_configCsd | CapSenseButton_CMOD_PRECHARGE_CONFIG);
                    #else
                        CY_SET_REG32(CapSenseButton_CONFIG_PTR, CapSenseButton_configCsd | CapSenseButton_CTANK_PRECHARGE_CONFIG);
                    #endif
                #else
                    /* Connect Vref Buffer to AMUX bus. Third-generation HW block is enabled */
                    CY_SET_REG32(CapSenseButton_CONFIG_PTR, CapSenseButton_configCsd | CapSenseButton_CTANK_PRECHARGE_CONFIG_CSD_EN);
                #endif

                /* Update Scan Counter */
                CapSenseButton_dsRam.scanCounter++;

                /* Sensor is totally scanned. Reset BUSY flag */
                CapSenseButton_dsRam.status &= ~(CapSenseButton_SW_STS_BUSY | CapSenseButton_WDGT_SW_STS_BUSY);
            }
        #endif /* (CapSenseButton_ENABLE == CapSenseButton_MULTI_FREQ_SCAN_EN) */

        #ifdef CapSenseButton_EXIT_CALLBACK
            CapSenseButton_ExitCallback();
        #endif /* CapSenseButton_EXIT_CALLBACK */
    }


    /*******************************************************************************
    * Function Name: CapSenseButton_CSDPostMultiScan
    ****************************************************************************//**
    *
    * \brief
    *  This is an internal ISR function for the multiple-sensor scanning
    *  implementation.
    *
    * \details
    *  This ISR handler is triggered when the user calls the
    *  CapSenseButton_Scan() or CapSenseButton_ScanAllWidgets() APIs.
    *
    *  The following tasks are performed:
    *    1. Disable the CSD interrupt.
    *    2. Read the Counter register and update the data structure with raw data.
    *    3. Connect the Vref buffer to the AMUX bus.
    *    4. Disable the CSD block (after the widget has been scanned).
    *    5. Update the Scan Counter.
    *    6. Reset the BUSY flag.
    *    7. Enable the CSD interrupt.
    *
    *  The ISR handler initializes scanning for the previous sensor when the
    *  widget has more than one sensor.
    *  The ISR handler initializes scanning for the next widget when the
    *  CapSenseButton_ScanAllWidgets() APIs are called and the project has
    *  more than one widget.
    *  The ISR handler changes the IMO and initializes scanning for the next
    *  frequency channels when multi-frequency scanning is enabled.
    *
    *  This function has two Macro Callbacks that allow calling the user
    *  code from macros specified in Component's generated code. Refer to the
    *  \ref group_c_macrocallbacks section of the PSoC Creator User Guide
    *  for details.
    *
    *******************************************************************************/
    CY_ISR(CapSenseButton_CSDPostMultiScan)
    {
        /* Declare and initialize ptr to sensor IO structure to appropriate address        */
        CapSenseButton_FLASH_IO_STRUCT const *curSnsIOPtr = (CapSenseButton_FLASH_IO_STRUCT const *)
                                                          CapSenseButton_dsFlash.wdgtArray[CapSenseButton_widgetIndex].ptr2SnsFlash
                                                          + CapSenseButton_sensorIndex;

        #ifdef CapSenseButton_ENTRY_CALLBACK
            CapSenseButton_EntryCallback();
        #endif /* CapSenseButton_ENTRY_CALLBACK */

        /* Clear pending interrupt */
        CY_SET_REG32(CapSenseButton_INTR_PTR, CapSenseButton_INTR_SET_MASK);

         /* Read Rawdata */
        CapSenseButton_SsCSDPostScan();

        /* Connect Vref Buffer to AMUX bus */
        CY_SET_REG32(CapSenseButton_CONFIG_PTR, CapSenseButton_configCsd | CapSenseButton_CTANK_PRECHARGE_CONFIG_CSD_EN);

        #if (CapSenseButton_ENABLE == CapSenseButton_MULTI_FREQ_SCAN_EN)
            /* Disable sensor when all frequency channels are scanned */
            if (CapSenseButton_FREQ_CHANNEL_2 == CapSenseButton_scanFreqIndex)
            {
                /* Disable sensor */
                CapSenseButton_CSDDisconnectSns(curSnsIOPtr);
            }
        #else
            /* Disable sensor */
            CapSenseButton_CSDDisconnectSns(curSnsIOPtr);
        #endif /* (CapSenseButton_ENABLE == CapSenseButton_MULTI_FREQ_SCAN_EN) */

        #if (CapSenseButton_ENABLE == CapSenseButton_MULTI_FREQ_SCAN_EN)
            if (CapSenseButton_FREQ_CHANNEL_2 > CapSenseButton_scanFreqIndex)
            {
                 /* Scan the next channel */
                CapSenseButton_SsNextFrequencyScan();
            }
            else
            {
                 /* All channels are scanned. Set IMO to zero channel */
                CapSenseButton_scanFreqIndex = CapSenseButton_FREQ_CHANNEL_0;

                #if (CapSenseButton_MFS_IMO == CapSenseButton_MFS_METHOD)
                    CapSenseButton_SsChangeImoFreq(CapSenseButton_FREQ_CHANNEL_0);
                #else
                    CapSenseButton_SsChangeClkFreq(CapSenseButton_FREQ_CHANNEL_0);
                #endif /* (CapSenseButton_MFS_IMO == CapSenseButton_MFS_METHOD) */

                 /* Scan the next sensor */
                CapSenseButton_SsCSDInitNextScan();
            }
        #else
            /* Scan the next sensor */
            CapSenseButton_SsCSDInitNextScan();
        #endif /* (CapSenseButton_ENABLE == CapSenseButton_MULTI_FREQ_SCAN_EN) */

        #ifdef CapSenseButton_EXIT_CALLBACK
            CapSenseButton_ExitCallback();
        #endif /* CapSenseButton_EXIT_CALLBACK */
    }


    #if (CapSenseButton_ENABLE == CapSenseButton_CSD_GANGED_SNS_EN)
    /*******************************************************************************
    * Function Name: CapSenseButton_CSDPostMultiScanGanged
    ****************************************************************************//**
    *
    * \brief
    *  This is an internal ISR function for the multiple-sensor scanning
    *  implementation for ganged sensors.
    *
    * \details
    *  This ISR handler is triggered when the user calls the
    *  CapSenseButton_Scan() API for a ganged sensor or the
    *  CapSenseButton_ScanAllWidgets() API in the project with ganged sensors.
    *
    *  The following tasks are performed:
    *    1. Disable the CSD interrupt.
    *    2. Read the Counter register and update the data structure with raw data.
    *    3. Connect the Vref buffer to the AMUX bus.
    *    4. Disable the CSD block (after the widget has been scanned).
    *    5. Update the Scan Counter.
    *    6. Reset the BUSY flag.
    *    7. Enable the CSD interrupt.
    *
    *  The ISR handler initializes scanning for the previous sensor when the
    *  widget has more than one sensor.
    *  The ISR handler initializes scanning for the next widget when the
    *  CapSenseButton_ScanAllWidgets() APIs are called and the project has
    *  more than one widget.
    *  The ISR handler changes the IMO and initializes scanning for the next
    *  frequency channels when multi-frequency scanning is enabled.
    *
    *  This function has two Macro Callbacks that allow calling the user
    *  code from macros specified in Component's generated code. Refer to the
    *  \ref group_c_macrocallbacks section of the PSoC Creator User Guide
    *  for details.
    *
    *******************************************************************************/
    CY_ISR(CapSenseButton_CSDPostMultiScanGanged)
    {
        #ifdef CapSenseButton_ENTRY_CALLBACK
            CapSenseButton_EntryCallback();
        #endif /* CapSenseButton_ENTRY_CALLBACK */

        /* Clear pending interrupt */
        CY_SET_REG32(CapSenseButton_INTR_PTR, CapSenseButton_INTR_SET_MASK);

         /* Read Rawdata */
        CapSenseButton_SsCSDPostScan();

        /* Connect Vref Buffer to AMUX bus */
        CY_SET_REG32(CapSenseButton_CONFIG_PTR, CapSenseButton_configCsd | CapSenseButton_CTANK_PRECHARGE_CONFIG_CSD_EN);

        #if (CapSenseButton_ENABLE == CapSenseButton_MULTI_FREQ_SCAN_EN)
            if (CapSenseButton_FREQ_CHANNEL_2 == CapSenseButton_scanFreqIndex)
            {
                CapSenseButton_SsCSDDisconnectSnsExt((uint32)CapSenseButton_widgetIndex, (uint32)CapSenseButton_sensorIndex);
            }
        #else
            CapSenseButton_SsCSDDisconnectSnsExt((uint32)CapSenseButton_widgetIndex, (uint32)CapSenseButton_sensorIndex);
        #endif /* (CapSenseButton_ENABLE == CapSenseButton_MULTI_FREQ_SCAN_EN) */

        #if (CapSenseButton_ENABLE == CapSenseButton_MULTI_FREQ_SCAN_EN)
            if (CapSenseButton_FREQ_CHANNEL_2 > CapSenseButton_scanFreqIndex)
            {
                 /* Scan the next channel */
                CapSenseButton_SsNextFrequencyScan();
            }
            else
            {
                /* All channels are scanned. Set IMO to zero channel */
                CapSenseButton_scanFreqIndex = CapSenseButton_FREQ_CHANNEL_0;

                #if (CapSenseButton_MFS_IMO == CapSenseButton_MFS_METHOD)
                    CapSenseButton_SsChangeImoFreq(CapSenseButton_FREQ_CHANNEL_0);
                #else
                    CapSenseButton_SsChangeClkFreq(CapSenseButton_FREQ_CHANNEL_0);
                #endif /* (CapSenseButton_MFS_IMO == CapSenseButton_MFS_METHOD) */

                 /* Scan the next sensor */
                CapSenseButton_SsCSDInitNextScan();
            }
        #else
             /* Scan the next sensor */
            CapSenseButton_SsCSDInitNextScan();
        #endif /* (CapSenseButton_ENABLE == CapSenseButton_MULTI_FREQ_SCAN_EN) */

        #ifdef CapSenseButton_EXIT_CALLBACK
            CapSenseButton_ExitCallback();
        #endif /* CapSenseButton_EXIT_CALLBACK */
    }
    #endif /* (CapSenseButton_ENABLE == CapSenseButton_CSD_GANGED_SNS_EN) */

#endif /* (CapSenseButton_ENABLE == CapSenseButton_CSDV2) */

#endif /* ((CapSenseButton_ENABLE == CapSenseButton_CSD_EN) || (CapSenseButton_ENABLE == CapSenseButton_CSD_CSX_EN)) */

/** \}
 * \endcond */


#if ((CapSenseButton_ENABLE == CapSenseButton_CSD_EN) || (CapSenseButton_ENABLE == CapSenseButton_CSD_CSX_EN))

/*******************************************************************************
* Function Name: CapSenseButton_SsCSDPostScan
****************************************************************************//**
*
* \brief
*   This function reads rawdata and releases required HW resources after scan.
*
* \details
*   This function performs following tasks after scan:
*   - Reads SlotResult from Raw Counter;
*   - Inits bad Conversions number;
*   - Disconnects Vrefhi from AMUBUF positive input;
*   - Disconnects AMUBUF output from CSDBUSB with sych PHI2+HSCMP;
*   - Opens HCBV and HCBG switches.
*
*******************************************************************************/
static void CapSenseButton_SsCSDPostScan(void)
{
    #if (CapSenseButton_ENABLE == CapSenseButton_CSDV2)

        uint32 tmpRawData;
        uint32 tmpMaxCount;
        CapSenseButton_RAM_WD_BASE_STRUCT const *ptrWdgt = (CapSenseButton_RAM_WD_BASE_STRUCT *)
                                            CapSenseButton_dsFlash.wdgtArray[CapSenseButton_widgetIndex].ptr2WdgtRam;

        /* Read SlotResult from Raw Counter */
        tmpRawData = CapSenseButton_RESULT_VAL1_VALUE_MASK & CY_GET_REG32(CapSenseButton_COUNTER_PTR);

        tmpMaxCount = ((1uL << ptrWdgt->resolution) - 1uL);
        if(tmpRawData < tmpMaxCount)
        {
            CapSenseButton_curRamSnsPtr->raw[CapSenseButton_scanFreqIndex] = LO16(tmpRawData);
        }
        else
        {
            CapSenseButton_curRamSnsPtr->raw[CapSenseButton_scanFreqIndex] = LO16(tmpMaxCount);
        }

        #if (CapSenseButton_ENABLE == CapSenseButton_CSD_NOISE_METRIC_EN)
            /* Init bad Conversions number */
            CapSenseButton_badConversionsNum = CapSenseButton_BAD_CONVERSIONS_NUM;
        #endif /* (CapSenseButton_ENABLE == CapSenseButton_CSD_NOISE_METRIC_EN) */

        #if (CapSenseButton_ENABLE == CapSenseButton_CSD_SHIELD_EN)
            /* Open HCBV and HCBG switches */
            CY_SET_REG32(CapSenseButton_SW_SHIELD_SEL_PTR, CapSenseButton_SW_SHIELD_SEL_SW_HCBV_STATIC_OPEN |
                                                             CapSenseButton_SW_SHIELD_SEL_SW_HCBG_STATIC_OPEN);
        #endif /* (CapSenseButton_ENABLE == CapSenseButton_CSD_SHIELD_EN) */

    #else

        /* Read SlotResult from Raw Counter */
       CapSenseButton_curRamSnsPtr->raw[CapSenseButton_scanFreqIndex] = (uint16)CY_GET_REG32(CapSenseButton_COUNTER_PTR);

    #endif /* (CapSenseButton_ENABLE == CapSenseButton_CSDV2) */
}


/*******************************************************************************
* Function Name: CapSenseButton_SsCSDInitNextScan
****************************************************************************//**
*
* \brief
*   This function initializes the next sensor scan.
*
* \details
*   The function increments the sensor index, updates sense clock for matrix
*   or touchpad widgets only, sets up Compensation IDAC, enables the sensor and
*   scans it. When all the sensors are scanned it continues to set up the next widget
*   until all the widgets are scanned. The CSD block is disabled when all the widgets are
*   scanned.
*
*******************************************************************************/
static void CapSenseButton_SsCSDInitNextScan(void)
{
    /* Declare and initialize ptr to widget and sensor structures to appropriate address */
    #if (((CapSenseButton_ENABLE == CapSenseButton_CSD_IDAC_COMP_EN) || \
          (CapSenseButton_CSD_MATRIX_WIDGET_EN || CapSenseButton_CSD_TOUCHPAD_WIDGET_EN)) || \
          (((CapSenseButton_DISABLE == CapSenseButton_CSD_COMMON_SNS_CLK_EN) && \
          (CapSenseButton_ENABLE == CapSenseButton_CSDV2) && \
          (CapSenseButton_CSD_MATRIX_WIDGET_EN || CapSenseButton_CSD_TOUCHPAD_WIDGET_EN))))
        CapSenseButton_RAM_WD_BASE_STRUCT *ptrWdgt = (CapSenseButton_RAM_WD_BASE_STRUCT *)
                                                        CapSenseButton_dsFlash.wdgtArray[CapSenseButton_widgetIndex].ptr2WdgtRam;
    #endif

    /* Check if all the sensors are scanned in widget */
    if (((uint8)CapSenseButton_dsFlash.wdgtArray[(CapSenseButton_widgetIndex)].totalNumSns - 1u) > CapSenseButton_sensorIndex)
    {
        /* Increment sensor index to configure next sensor in widget */
        CapSenseButton_sensorIndex++;

        /* Update global pointer to CapSenseButton_RAM_SNS_STRUCT to current sensor  */
        CapSenseButton_curRamSnsPtr = (CapSenseButton_RAM_SNS_STRUCT *)
                                                  CapSenseButton_dsFlash.wdgtArray[CapSenseButton_widgetIndex].ptr2SnsRam
                                                  + CapSenseButton_sensorIndex;

        /* Configure clock divider to row or column */
        #if ((CapSenseButton_DISABLE == CapSenseButton_CSD_COMMON_SNS_CLK_EN) && \
             (CapSenseButton_CSD_MATRIX_WIDGET_EN || CapSenseButton_CSD_TOUCHPAD_WIDGET_EN))
            CapSenseButton_SsCSDConfigClock();

            #if (CapSenseButton_ENABLE == CapSenseButton_CSDV2)
                 /* Set up scanning resolution */
                CapSenseButton_SsCSDCalculateScanDuration(ptrWdgt);
            #endif
        #endif

        /* Setup Compensation IDAC for next sensor in widget */
        #if ((CapSenseButton_ENABLE == CapSenseButton_CSD_IDAC_COMP_EN) || \
             (CapSenseButton_CSD_MATRIX_WIDGET_EN || CapSenseButton_CSD_TOUCHPAD_WIDGET_EN))
            CapSenseButton_SsCSDSetUpIdacs(ptrWdgt);
        #endif /* ((CapSenseButton_ENABLE == CapSenseButton_CSD_IDAC_COMP_EN) || \
                   (CapSenseButton_CSD_MATRIX_WIDGET_EN || CapSenseButton_CSD_TOUCHPAD_WIDGET_EN)) */

        /* Enable sensor */
        CapSenseButton_SsCSDConnectSensorExt((uint32)CapSenseButton_widgetIndex, (uint32)CapSenseButton_sensorIndex);

        /* Proceed scanning */
        CapSenseButton_SsCSDStartSample();
    }
    /* Call scan next widget API if requested, if not, complete the scan  */
    else
    {
        CapSenseButton_sensorIndex = 0u;

        /* Current widget is totally scanned. Reset WIDGET BUSY flag */
        CapSenseButton_dsRam.status &= ~CapSenseButton_WDGT_SW_STS_BUSY;

        /* Check if all the widgets have been scanned */
        if (CapSenseButton_ENABLE == CapSenseButton_requestScanAllWidget)
        {
            /* Configure and begin scanning next widget */
            CapSenseButton_SsPostAllWidgetsScan();
        }
        else
        {
            #if (CapSenseButton_ENABLE == CapSenseButton_BLOCK_OFF_AFTER_SCAN_EN)
                #if (CapSenseButton_ENABLE == CapSenseButton_CSDV2)
                    /* Disable the CSD block */
                    CY_SET_REG32(CapSenseButton_CONFIG_PTR, CapSenseButton_configCsd);
                #else
                    /* Disable the CSD block. Connect Vref Buffer to AMUX bus */
                    #if ((CapSenseButton_CSH_PRECHARGE_IO_BUF == CapSenseButton_CSD_CSH_PRECHARGE_SRC) &&\
                         (CapSenseButton_ENABLE == CapSenseButton_CSD_SHIELD_TANK_EN))
                        CY_SET_REG32(CapSenseButton_CONFIG_PTR, CapSenseButton_configCsd | CapSenseButton_CMOD_PRECHARGE_CONFIG);
                    #else
                        CY_SET_REG32(CapSenseButton_CONFIG_PTR, CapSenseButton_configCsd | CapSenseButton_CTANK_PRECHARGE_CONFIG);
                    #endif
                #endif
            #endif

            /* All widgets are totally scanned. Reset BUSY flag */
            CapSenseButton_dsRam.status &= ~CapSenseButton_SW_STS_BUSY;

            /* Update scan Counter */
            CapSenseButton_dsRam.scanCounter++;
        }
    }
}

#if (CapSenseButton_ENABLE == CapSenseButton_MULTI_FREQ_SCAN_EN)
    /*******************************************************************************
    * Function Name: CapSenseButton_SsNextFrequencyScan
    ****************************************************************************//**
    *
    * \brief
    *   This function scans the sensor on the next frequency channel.
    *
    * \details
    *   The function increments the frequency channel, changes IMO and initializes
    *   the scanning process of the same sensor.
    *
    *******************************************************************************/
    static void CapSenseButton_SsNextFrequencyScan(void)
    {
        CapSenseButton_RAM_WD_BASE_STRUCT const *ptrWdgt = (CapSenseButton_RAM_WD_BASE_STRUCT *)
                                                        CapSenseButton_dsFlash.wdgtArray[CapSenseButton_widgetIndex].ptr2WdgtRam;

        CapSenseButton_scanFreqIndex++;

        /* Set Immunity */
        #if (CapSenseButton_MFS_IMO == CapSenseButton_MFS_METHOD)
            CapSenseButton_SsChangeImoFreq((uint32)CapSenseButton_scanFreqIndex);
        #else
            CapSenseButton_SsChangeClkFreq((uint32)CapSenseButton_scanFreqIndex);
        #endif /* (CapSenseButton_MFS_IMO == CapSenseButton_MFS_METHOD) */

        /* Update IDAC registers */
        CapSenseButton_SsCSDSetUpIdacs(ptrWdgt);

        /* Proceed scanning */
        CapSenseButton_SsCSDStartSample();
    }
#endif /* (CapSenseButton_ENABLE == CapSenseButton_MULTI_FREQ_SCAN_EN) */

#endif /* ((CapSenseButton_ENABLE == CapSenseButton_CSD_EN) || (CapSenseButton_ENABLE == CapSenseButton_CSD_CSX_EN)) */


/* [] END OF FILE */
