/***************************************************************************//**
* \file CapSenseProx_INT.c
* \version 7.0
*
* \brief
*   This file contains the source code for implementation of the Component's
*   Interrupt Service Routine (ISR).
*
* \see CapSenseProx v7.0 Datasheet
*
*//*****************************************************************************
* Copyright (2016-2019), Cypress Semiconductor Corporation.
********************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress) and is
* protected by and subject to worldwide patent protection (United States and
* foreign), United States copyright laws and international treaty provisions.
* Cypress hereby grants to licensee a personal, non-exclusive, non-transferable
* license to copy, use, modify, create derivative works of, and compile the
* Cypress Source Code and derivative works for the sole purpose of creating
* custom software in support of licensee product to be used only in conjunction
* with a Cypress integrated circuit as specified in the applicable agreement.
* Any reproduction, modification, translation, compilation, or representation of
* this software except as specified above is prohibited without the express
* written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the
* materials described herein. Cypress does not assume any liability arising out
* of the application or use of any product or circuit described herein. Cypress
* does not authorize its products for use as critical components in life-support
* systems where a malfunction or failure may reasonably be expected to result in
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of
* such use and in doing so indemnifies Cypress against all charges. Use may be
* limited by and subject to the applicable Cypress software license agreement.
*******************************************************************************/
#include "cytypes.h"
#include "cyfitter.h"
#include "CapSenseProx_Configuration.h"
#include "CapSenseProx_Structure.h"
#include "CapSenseProx_Sensing.h"
#if (CapSenseProx_ENABLE == CapSenseProx_CSD_EN)
    #include "CapSenseProx_SensingCSD_LL.h"
#endif /* (CapSenseProx_ENABLE == CapSenseProx_CSD_EN) */
#include "cyapicallbacks.h"

/*******************************************************************************
* Static Function Prototypes
*******************************************************************************/
/**
* \cond SECTION_C_INTERNAL
* \addtogroup group_c_internal
* \{
*/

#if (((CapSenseProx_ENABLE == CapSenseProx_CSD_EN) || (CapSenseProx_ENABLE == CapSenseProx_CSD_CSX_EN)) && \
     (CapSenseProx_ENABLE == CapSenseProx_MULTI_FREQ_SCAN_EN))
    static void CapSenseProx_SsNextFrequencyScan(void);
#endif /* (((CapSenseProx_ENABLE == CapSenseProx_CSD_EN) || (CapSenseProx_ENABLE == CapSenseProx_CSD_CSX_EN)) && \
            (CapSenseProx_ENABLE == CapSenseProx_MULTI_FREQ_SCAN_EN)) */

#if ((CapSenseProx_ENABLE == CapSenseProx_CSD_EN) || (CapSenseProx_ENABLE == CapSenseProx_CSD_CSX_EN))
    static void CapSenseProx_SsCSDPostScan(void);
    static void CapSenseProx_SsCSDInitNextScan(void);
#endif /* ((CapSenseProx_ENABLE == CapSenseProx_CSD_EN) || (CapSenseProx_ENABLE == CapSenseProx_CSD_CSX_EN)) */
/** \}
* \endcond */


/**
* \cond SECTION_C_INTERRUPT
* \addtogroup group_c_interrupt
* \{
*/


#if ((CapSenseProx_ENABLE == CapSenseProx_CSD_EN) || (CapSenseProx_ENABLE == CapSenseProx_CSD_CSX_EN))

#if (CapSenseProx_ENABLE == CapSenseProx_CSDV2)
    /* Fourth-generation HW block part */

    /*******************************************************************************
    * Function Name: CapSenseProx_CSDPostSingleScan
    ****************************************************************************//**
    *
    * \brief
    *  This is an internal ISR function for the single-sensor scanning implementation.
    *
    * \details
    *  This ISR handler is triggered when the user calls the
    *  CapSenseProx_CSDScanExt() function.
    *
    *  The following tasks are performed for Third-generation HW block:
    *    1. Disable the CSD interrupt.
    *    2. Read the Counter register and update the data structure with raw data.
    *    3. Connect the Vref buffer to the AMUX bus.
    *    4. Update the Scan Counter.
    *    5. Reset the BUSY flag.
    *    6. Enable the CSD interrupt.
    *
    *  The following tasks are performed for Fourth-generation HW block:
    *    1. Check if the raw data is not noisy.
    *    2. Read the Counter register and update the data structure with raw data.
    *    3. Configure and start the scan for the next frequency if the
    *      multi-frequency is enabled.
    *    4. Update the Scan Counter.
    *    5. Reset the BUSY flag.
    *    6. Enable the CSD interrupt.
    *
    *  The ISR handler changes the IMO and initializes scanning for the next frequency
    *  channels when multi-frequency scanning is enabled.
    *
    *  This function has two Macro Callbacks that allow calling the user code
    *  from macros specified in Component's generated code. Refer to the
    *  \ref group_c_macrocallbacks section of the PSoC Creator User Guide
    *  for details.
    *
    *******************************************************************************/
    CY_ISR(CapSenseProx_CSDPostSingleScan)
    {
        #ifdef CapSenseProx_ENTRY_CALLBACK
            CapSenseProx_EntryCallback();
        #endif /* CapSenseProx_ENTRY_CALLBACK */

        /* Clear pending interrupt */
        CY_SET_REG32(CapSenseProx_INTR_PTR, CapSenseProx_INTR_SET_MASK);

        #if (CapSenseProx_ENABLE == CapSenseProx_CSD_NOISE_METRIC_EN)
            if ((CapSenseProx_CSD_NOISE_METRIC_TH < ((CY_GET_REG32(CapSenseProx_RESULT_VAL1_PTR) &
                                                        CapSenseProx_RESULT_VAL1_BAD_CONVS_MASK) >>
                                                        CapSenseProx_RESULT_VAL1_BAD_CONVS_SHIFT)) &&
                                                        (0u < CapSenseProx_badConversionsNum))
            {
                /* Decrement bad conversions number */
                CapSenseProx_badConversionsNum--;

                /* Start the re-scan */
                CY_SET_REG32(CapSenseProx_SEQ_START_PTR, CapSenseProx_SEQ_START_AZ0_SKIP_MASK |
                                                             CapSenseProx_SEQ_START_AZ1_SKIP_MASK |
                                                             CapSenseProx_SEQ_START_START_MASK);
            }
            else
            {
        #endif /* (CapSenseProx_ENABLE == CapSenseProx_CSD_NOISE_METRIC_EN) */

            CapSenseProx_SsCSDPostScan();

            #if (CapSenseProx_ENABLE == CapSenseProx_MULTI_FREQ_SCAN_EN)
                if (CapSenseProx_FREQ_CHANNEL_2 > CapSenseProx_scanFreqIndex)
                {
                    /* Scan the next channel */
                    CapSenseProx_SsNextFrequencyScan();
                }
                else
                {
                    /* All channels are scanned. Set IMO to zero channel */
                    CapSenseProx_scanFreqIndex = CapSenseProx_FREQ_CHANNEL_0;

                    #if (CapSenseProx_MFS_IMO == CapSenseProx_MFS_METHOD)
                        CapSenseProx_SsChangeImoFreq(CapSenseProx_FREQ_CHANNEL_0);
                    #else
                        CapSenseProx_SsChangeClkFreq(CapSenseProx_FREQ_CHANNEL_0);
                    #endif /* (CapSenseProx_MFS_IMO == CapSenseProx_MFS_METHOD) */

                    #if (CapSenseProx_ENABLE == CapSenseProx_BLOCK_OFF_AFTER_SCAN_EN)
                        /* Disable Fourth-generation HW block */
                        CY_SET_REG32(CapSenseProx_CONFIG_PTR, CapSenseProx_configCsd);
                    #endif /* (CapSenseProx_ENABLE == CapSenseProx_BLOCK_OFF_AFTER_SCAN_EN) */

                    /* Update Scan Counter */
                    CapSenseProx_dsRam.scanCounter++;

                    /* Sensor is totally scanned. Reset BUSY flag */
                    CapSenseProx_dsRam.status &= ~(CapSenseProx_SW_STS_BUSY | CapSenseProx_WDGT_SW_STS_BUSY);
                }
            #else
                {
                    #if (CapSenseProx_ENABLE == CapSenseProx_BLOCK_OFF_AFTER_SCAN_EN)
                        /* Disable Fourth-generation HW block */
                        CY_SET_REG32(CapSenseProx_CONFIG_PTR, CapSenseProx_configCsd);
                    #endif /* (CapSenseProx_ENABLE == CapSenseProx_BLOCK_OFF_AFTER_SCAN_EN) */

                    /* Update Scan Counter */
                    CapSenseProx_dsRam.scanCounter++;

                    /* Sensor is totally scanned. Reset BUSY flag */
                    CapSenseProx_dsRam.status &= ~(CapSenseProx_SW_STS_BUSY | CapSenseProx_WDGT_SW_STS_BUSY);
                }
            #endif /* (CapSenseProx_ENABLE == CapSenseProx_MULTI_FREQ_SCAN_EN) */
    #if (CapSenseProx_ENABLE == CapSenseProx_CSD_NOISE_METRIC_EN)
        }
    #endif /* (CapSenseProx_ENABLE == CapSenseProx_CSD_NOISE_METRIC_EN) */

        #ifdef CapSenseProx_EXIT_CALLBACK
            CapSenseProx_ExitCallback();
        #endif /* CapSenseProx_EXIT_CALLBACK */
    }


    /*******************************************************************************
    * Function Name: CapSenseProx_CSDPostMultiScan
    ****************************************************************************//**
    *
    * \brief
    *  This is an internal ISR function for the multiple-sensor scanning
    *  implementation.
    *
    * \details
    *  This ISR handler is triggered when the user calls the
    *  CapSenseProx_Scan() or CapSenseProx_ScanAllWidgets() APIs.
    *
    *  The following tasks are performed:
    *    1. Disable the CSD interrupt.
    *    2. Read the Counter register and update the data structure with raw data.
    *    3. Connect the Vref buffer to the AMUX bus.
    *    4. Disable the CSD block (after the widget has been scanned).
    *    5. Update the Scan Counter.
    *    6. Reset the BUSY flag.
    *    7. Enable the CSD interrupt.
    *
    *  The ISR handler initializes scanning for the previous sensor when the
    *  widget has more than one sensor.
    *  The ISR handler initializes scanning for the next widget when the
    *  CapSenseProx_ScanAllWidgets() APIs are called and the project has
    *  more than one widget.
    *  The ISR handler changes the IMO and initializes scanning for the next
    *  frequency channels when multi-frequency scanning is enabled.
    *
    *  This function has two Macro Callbacks that allow calling the user
    *  code from macros specified in Component's generated code. Refer to the
    *  \ref group_c_macrocallbacks section of the PSoC Creator User Guide
    *  for details.
    *
    *******************************************************************************/
    CY_ISR(CapSenseProx_CSDPostMultiScan)
    {
        /* Declare and initialize ptr to sensor IO structure to appropriate address */
        CapSenseProx_FLASH_IO_STRUCT const *curSnsIOPtr = (CapSenseProx_FLASH_IO_STRUCT const *)
                                                          CapSenseProx_dsFlash.wdgtArray[CapSenseProx_widgetIndex].ptr2SnsFlash
                                                          + CapSenseProx_sensorIndex;

        #ifdef CapSenseProx_ENTRY_CALLBACK
            CapSenseProx_EntryCallback();
        #endif /* CapSenseProx_ENTRY_CALLBACK */

        /* Clear pending interrupt */
        CY_SET_REG32(CapSenseProx_INTR_PTR, CapSenseProx_INTR_SET_MASK);

        #if (CapSenseProx_ENABLE == CapSenseProx_CSD_NOISE_METRIC_EN)
            if ((CapSenseProx_CSD_NOISE_METRIC_TH < ((CY_GET_REG32(CapSenseProx_RESULT_VAL1_PTR) &
                                                      CapSenseProx_RESULT_VAL1_BAD_CONVS_MASK) >>
                                                      CapSenseProx_RESULT_VAL1_BAD_CONVS_SHIFT)) &&
                                                      (0u < CapSenseProx_badConversionsNum))
            {
                /* Decrement bad conversions number */
                CapSenseProx_badConversionsNum--;

                /* Start the re-scan */
                CY_SET_REG32(CapSenseProx_SEQ_START_PTR, CapSenseProx_SEQ_START_AZ0_SKIP_MASK |
                                                             CapSenseProx_SEQ_START_AZ1_SKIP_MASK |
                                                             CapSenseProx_SEQ_START_START_MASK);
            }
            else
            {
        #endif /* (CapSenseProx_ENABLE == CapSenseProx_CSD_NOISE_METRIC_EN) */

            CapSenseProx_SsCSDPostScan();

            /* Disable sensor when all frequency channels are scanned */
            #if (CapSenseProx_ENABLE == CapSenseProx_MULTI_FREQ_SCAN_EN)
                if (CapSenseProx_FREQ_CHANNEL_2 == CapSenseProx_scanFreqIndex)
            #endif /* (CapSenseProx_ENABLE == CapSenseProx_MULTI_FREQ_SCAN_EN) */
            {
                /* Disable sensor */
                CapSenseProx_CSDDisconnectSns(curSnsIOPtr);
            }

            #if (CapSenseProx_ENABLE == CapSenseProx_MULTI_FREQ_SCAN_EN)
                if (CapSenseProx_FREQ_CHANNEL_2 > CapSenseProx_scanFreqIndex)
                {
                     /* Scan the next channel */
                    CapSenseProx_SsNextFrequencyScan();
                }
                else
                {
                     /* All channels are scanned. Set IMO to zero channel */
                    CapSenseProx_scanFreqIndex = CapSenseProx_FREQ_CHANNEL_0;

                    #if (CapSenseProx_MFS_IMO == CapSenseProx_MFS_METHOD)
                        CapSenseProx_SsChangeImoFreq(CapSenseProx_FREQ_CHANNEL_0);
                    #else
                        CapSenseProx_SsChangeClkFreq(CapSenseProx_FREQ_CHANNEL_0);
                    #endif /* (CapSenseProx_MFS_IMO == CapSenseProx_MFS_METHOD) */

                     /* Scan the next sensor */
                    CapSenseProx_SsCSDInitNextScan();
                }
            #else
                /* Scan the next sensor */
                CapSenseProx_SsCSDInitNextScan();
            #endif /* (CapSenseProx_ENABLE == CapSenseProx_MULTI_FREQ_SCAN_EN) */
        #if (CapSenseProx_ENABLE == CapSenseProx_CSD_NOISE_METRIC_EN)
            }
        #endif /* (CapSenseProx_ENABLE == CapSenseProx_CSD_NOISE_METRIC_EN) */

        #ifdef CapSenseProx_EXIT_CALLBACK
            CapSenseProx_ExitCallback();
        #endif /* CapSenseProx_EXIT_CALLBACK */
    }


    #if (CapSenseProx_ENABLE == CapSenseProx_CSD_GANGED_SNS_EN)
    /*******************************************************************************
    * Function Name: CapSenseProx_CSDPostMultiScanGanged
    ****************************************************************************//**
    *
    * \brief
    *  This is an internal ISR function for the multiple-sensor scanning
    *  implementation for ganged sensors.
    *
    * \details
    *  This ISR handler is triggered when the user calls the
    *  CapSenseProx_Scan() API for a ganged sensor or the
    *  CapSenseProx_ScanAllWidgets() API in the project with ganged sensors.
    *
    *  The following tasks are performed:
    *    1. Disable the CSD interrupt.
    *    2. Read the Counter register and update the data structure with raw data.
    *    3. Connect the Vref buffer to the AMUX bus.
    *    4. Disable the CSD block (after the widget has been scanned).
    *    5. Update the Scan Counter.
    *    6. Reset the BUSY flag.
    *    7. Enable the CSD interrupt.
    *
    *  The ISR handler initializes scanning for the previous sensor when the
    *  widget has more than one sensor.
    *  The ISR handler initializes scanning for the next widget when the
    *  CapSenseProx_ScanAllWidgets() APIs are called and the project has
    *  more than one widget.
    *  The ISR handler changes the IMO and initializes scanning for the next
    *  frequency channels when multi-frequency scanning is enabled.
    *
    *  This function has two Macro Callbacks that allow calling the user
    *  code from macros specified in Component's generated code. Refer to the
    *  \ref group_c_macrocallbacks section of the PSoC Creator User Guide
    *  for details.
    *
    *******************************************************************************/
    CY_ISR(CapSenseProx_CSDPostMultiScanGanged)
    {
        #ifdef CapSenseProx_ENTRY_CALLBACK
            CapSenseProx_EntryCallback();
        #endif /* CapSenseProx_ENTRY_CALLBACK */

        /* Clear pending interrupt */
        CY_SET_REG32(CapSenseProx_INTR_PTR, CapSenseProx_INTR_SET_MASK);

        #if (CapSenseProx_ENABLE == CapSenseProx_CSD_NOISE_METRIC_EN)
            if ((CapSenseProx_CSD_NOISE_METRIC_TH < ((CY_GET_REG32(CapSenseProx_RESULT_VAL1_PTR) &
                                                      CapSenseProx_RESULT_VAL1_BAD_CONVS_MASK) >>
                                                      CapSenseProx_RESULT_VAL1_BAD_CONVS_SHIFT)) &&
                                                      (0u < CapSenseProx_badConversionsNum))
            {
                /* Decrement bad conversions number */
                CapSenseProx_badConversionsNum--;

                /* Start the re-scan */
                CY_SET_REG32(CapSenseProx_SEQ_START_PTR, CapSenseProx_SEQ_START_AZ0_SKIP_MASK |
                                                             CapSenseProx_SEQ_START_AZ1_SKIP_MASK |
                                                             CapSenseProx_SEQ_START_START_MASK);
            }
            else
            {
        #endif /* (CapSenseProx_ENABLE == CapSenseProx_CSD_NOISE_METRIC_EN) */

            CapSenseProx_SsCSDPostScan();

            #if (CapSenseProx_ENABLE == CapSenseProx_MULTI_FREQ_SCAN_EN)
                if (CapSenseProx_FREQ_CHANNEL_2 == CapSenseProx_scanFreqIndex)
            #endif /* (CapSenseProx_ENABLE == CapSenseProx_MULTI_FREQ_SCAN_EN) */
            {
                CapSenseProx_SsCSDDisconnectSnsExt((uint32)CapSenseProx_widgetIndex, (uint32)CapSenseProx_sensorIndex);
            }

            #if (CapSenseProx_ENABLE == CapSenseProx_MULTI_FREQ_SCAN_EN)
                if (CapSenseProx_FREQ_CHANNEL_2 > CapSenseProx_scanFreqIndex)
                {
                     /* Scan the next channel */
                    CapSenseProx_SsNextFrequencyScan();
                }
                else
                {
                    /* All channels are scanned. Set IMO to zero channel */
                    CapSenseProx_scanFreqIndex = CapSenseProx_FREQ_CHANNEL_0;

                    #if (CapSenseProx_MFS_IMO == CapSenseProx_MFS_METHOD)
                        CapSenseProx_SsChangeImoFreq(CapSenseProx_FREQ_CHANNEL_0);
                    #else
                        CapSenseProx_SsChangeClkFreq(CapSenseProx_FREQ_CHANNEL_0);
                    #endif /* (CapSenseProx_MFS_IMO == CapSenseProx_MFS_METHOD) */

                     /* Scan the next sensor */
                    CapSenseProx_SsCSDInitNextScan();
                }
            #else
                 /* Scan the next sensor */
                CapSenseProx_SsCSDInitNextScan();
            #endif /* (CapSenseProx_ENABLE == CapSenseProx_MULTI_FREQ_SCAN_EN) */
        #if (CapSenseProx_ENABLE == CapSenseProx_CSD_NOISE_METRIC_EN)
            }
        #endif /* (CapSenseProx_ENABLE == CapSenseProx_CSD_NOISE_METRIC_EN) */

        #ifdef CapSenseProx_EXIT_CALLBACK
            CapSenseProx_ExitCallback();
        #endif /* CapSenseProx_EXIT_CALLBACK */
    }
    #endif /* (CapSenseProx_ENABLE == CapSenseProx_CSD_GANGED_SNS_EN) */

#else

    /* Third-generation HW block part */

    /*******************************************************************************
    * Function Name: CapSenseProx_CSDPostSingleScan
    ****************************************************************************//**
    *
    * \brief
    *  This is an internal ISR function for the single-sensor scanning implementation.
    *
    * \details
    *  This ISR handler is triggered when the user calls the
    *  CapSenseProx_CSDScanExt() function.
    *
    *  The following tasks are performed for Third-generation HW block:
    *    1. Disable the CSD interrupt.
    *    2. Read the Counter register and update the data structure with raw data.
    *    3. Connect the Vref buffer to the AMUX bus.
    *    4. Update the Scan Counter.
    *    5. Reset the BUSY flag.
    *    6. Enable the CSD interrupt.
    *
    *  The following tasks are performed for Fourth-generation HW block:
    *    1. Check if the raw data is not noisy.
    *    2. Read the Counter register and update the data structure with raw data.
    *    3. Configure and start the scan for the next frequency if the
    *      multi-frequency is enabled.
    *    4. Update the Scan Counter.
    *    5. Reset the BUSY flag.
    *    6. Enable the CSD interrupt.
    *
    *  The ISR handler changes the IMO and initializes scanning for the next frequency
    *  channels when multi-frequency scanning is enabled.
    *
    *  This function has two Macro Callbacks that allow calling the user code
    *  from macros specified in Component's generated code. Refer to the
    *  \ref group_c_macrocallbacks section of the PSoC Creator User Guide
    *  for details.
    *
    *******************************************************************************/
    CY_ISR(CapSenseProx_CSDPostSingleScan)
    {
        #ifdef CapSenseProx_ENTRY_CALLBACK
            CapSenseProx_EntryCallback();
        #endif /* CapSenseProx_ENTRY_CALLBACK */

        /* Clear pending interrupt */
        CY_SET_REG32(CapSenseProx_INTR_PTR, CapSenseProx_INTR_SET_MASK);

        /* Read Rawdata */
        CapSenseProx_SsCSDPostScan();

        #if (CapSenseProx_ENABLE == CapSenseProx_MULTI_FREQ_SCAN_EN)
            if (CapSenseProx_FREQ_CHANNEL_2 > CapSenseProx_scanFreqIndex)
            {
                /* Connect Vref Buffer to AMUX bus. Third-generation HW block is enabled */
                CY_SET_REG32(CapSenseProx_CONFIG_PTR, CapSenseProx_configCsd | CapSenseProx_CTANK_PRECHARGE_CONFIG_CSD_EN);

                CapSenseProx_SsNextFrequencyScan();
            }
            else
            {
                CapSenseProx_scanFreqIndex = CapSenseProx_FREQ_CHANNEL_0;

                #if (CapSenseProx_MFS_IMO == CapSenseProx_MFS_METHOD)
                    CapSenseProx_SsChangeImoFreq(CapSenseProx_FREQ_CHANNEL_0);
                #else
                    CapSenseProx_SsChangeClkFreq(CapSenseProx_FREQ_CHANNEL_0);
                #endif /* (CapSenseProx_MFS_IMO == CapSenseProx_MFS_METHOD) */

                #if (CapSenseProx_ENABLE == CapSenseProx_BLOCK_OFF_AFTER_SCAN_EN)
                    /* Disable Third-generation HW block. Connect Vref Buffer to AMUX bus */
                    #if ((CapSenseProx_CSH_PRECHARGE_IO_BUF == CapSenseProx_CSD_CSH_PRECHARGE_SRC) &&\
                         (CapSenseProx_ENABLE == CapSenseProx_CSD_SHIELD_TANK_EN))
                        CY_SET_REG32(CapSenseProx_CONFIG_PTR, CapSenseProx_configCsd | CapSenseProx_CMOD_PRECHARGE_CONFIG);
                    #else
                        CY_SET_REG32(CapSenseProx_CONFIG_PTR, CapSenseProx_configCsd | CapSenseProx_CTANK_PRECHARGE_CONFIG);
                    #endif
                #else
                    /* Connect Vref Buffer to AMUX bus. Third-generation HW block is enabled */
                    CY_SET_REG32(CapSenseProx_CONFIG_PTR, CapSenseProx_configCsd | CapSenseProx_CTANK_PRECHARGE_CONFIG_CSD_EN);
                #endif

                /* Update Scan Counter */
                CapSenseProx_dsRam.scanCounter++;

                /* Sensor is totally scanned. Reset BUSY flag */
                CapSenseProx_dsRam.status &= ~(CapSenseProx_SW_STS_BUSY | CapSenseProx_WDGT_SW_STS_BUSY);
            }
        #else
            {
                #if (CapSenseProx_ENABLE == CapSenseProx_BLOCK_OFF_AFTER_SCAN_EN)
                    /* Disable Third-generation HW block. Connect Vref Buffer to AMUX bus */
                    #if ((CapSenseProx_CSH_PRECHARGE_IO_BUF == CapSenseProx_CSD_CSH_PRECHARGE_SRC) &&\
                         (CapSenseProx_ENABLE == CapSenseProx_CSD_SHIELD_TANK_EN))
                        CY_SET_REG32(CapSenseProx_CONFIG_PTR, CapSenseProx_configCsd | CapSenseProx_CMOD_PRECHARGE_CONFIG);
                    #else
                        CY_SET_REG32(CapSenseProx_CONFIG_PTR, CapSenseProx_configCsd | CapSenseProx_CTANK_PRECHARGE_CONFIG);
                    #endif
                #else
                    /* Connect Vref Buffer to AMUX bus. Third-generation HW block is enabled */
                    CY_SET_REG32(CapSenseProx_CONFIG_PTR, CapSenseProx_configCsd | CapSenseProx_CTANK_PRECHARGE_CONFIG_CSD_EN);
                #endif

                /* Update Scan Counter */
                CapSenseProx_dsRam.scanCounter++;

                /* Sensor is totally scanned. Reset BUSY flag */
                CapSenseProx_dsRam.status &= ~(CapSenseProx_SW_STS_BUSY | CapSenseProx_WDGT_SW_STS_BUSY);
            }
        #endif /* (CapSenseProx_ENABLE == CapSenseProx_MULTI_FREQ_SCAN_EN) */

        #ifdef CapSenseProx_EXIT_CALLBACK
            CapSenseProx_ExitCallback();
        #endif /* CapSenseProx_EXIT_CALLBACK */
    }


    /*******************************************************************************
    * Function Name: CapSenseProx_CSDPostMultiScan
    ****************************************************************************//**
    *
    * \brief
    *  This is an internal ISR function for the multiple-sensor scanning
    *  implementation.
    *
    * \details
    *  This ISR handler is triggered when the user calls the
    *  CapSenseProx_Scan() or CapSenseProx_ScanAllWidgets() APIs.
    *
    *  The following tasks are performed:
    *    1. Disable the CSD interrupt.
    *    2. Read the Counter register and update the data structure with raw data.
    *    3. Connect the Vref buffer to the AMUX bus.
    *    4. Disable the CSD block (after the widget has been scanned).
    *    5. Update the Scan Counter.
    *    6. Reset the BUSY flag.
    *    7. Enable the CSD interrupt.
    *
    *  The ISR handler initializes scanning for the previous sensor when the
    *  widget has more than one sensor.
    *  The ISR handler initializes scanning for the next widget when the
    *  CapSenseProx_ScanAllWidgets() APIs are called and the project has
    *  more than one widget.
    *  The ISR handler changes the IMO and initializes scanning for the next
    *  frequency channels when multi-frequency scanning is enabled.
    *
    *  This function has two Macro Callbacks that allow calling the user
    *  code from macros specified in Component's generated code. Refer to the
    *  \ref group_c_macrocallbacks section of the PSoC Creator User Guide
    *  for details.
    *
    *******************************************************************************/
    CY_ISR(CapSenseProx_CSDPostMultiScan)
    {
        /* Declare and initialize ptr to sensor IO structure to appropriate address        */
        CapSenseProx_FLASH_IO_STRUCT const *curSnsIOPtr = (CapSenseProx_FLASH_IO_STRUCT const *)
                                                          CapSenseProx_dsFlash.wdgtArray[CapSenseProx_widgetIndex].ptr2SnsFlash
                                                          + CapSenseProx_sensorIndex;

        #ifdef CapSenseProx_ENTRY_CALLBACK
            CapSenseProx_EntryCallback();
        #endif /* CapSenseProx_ENTRY_CALLBACK */

        /* Clear pending interrupt */
        CY_SET_REG32(CapSenseProx_INTR_PTR, CapSenseProx_INTR_SET_MASK);

         /* Read Rawdata */
        CapSenseProx_SsCSDPostScan();

        /* Connect Vref Buffer to AMUX bus */
        CY_SET_REG32(CapSenseProx_CONFIG_PTR, CapSenseProx_configCsd | CapSenseProx_CTANK_PRECHARGE_CONFIG_CSD_EN);

        #if (CapSenseProx_ENABLE == CapSenseProx_MULTI_FREQ_SCAN_EN)
            /* Disable sensor when all frequency channels are scanned */
            if (CapSenseProx_FREQ_CHANNEL_2 == CapSenseProx_scanFreqIndex)
            {
                /* Disable sensor */
                CapSenseProx_CSDDisconnectSns(curSnsIOPtr);
            }
        #else
            /* Disable sensor */
            CapSenseProx_CSDDisconnectSns(curSnsIOPtr);
        #endif /* (CapSenseProx_ENABLE == CapSenseProx_MULTI_FREQ_SCAN_EN) */

        #if (CapSenseProx_ENABLE == CapSenseProx_MULTI_FREQ_SCAN_EN)
            if (CapSenseProx_FREQ_CHANNEL_2 > CapSenseProx_scanFreqIndex)
            {
                 /* Scan the next channel */
                CapSenseProx_SsNextFrequencyScan();
            }
            else
            {
                 /* All channels are scanned. Set IMO to zero channel */
                CapSenseProx_scanFreqIndex = CapSenseProx_FREQ_CHANNEL_0;

                #if (CapSenseProx_MFS_IMO == CapSenseProx_MFS_METHOD)
                    CapSenseProx_SsChangeImoFreq(CapSenseProx_FREQ_CHANNEL_0);
                #else
                    CapSenseProx_SsChangeClkFreq(CapSenseProx_FREQ_CHANNEL_0);
                #endif /* (CapSenseProx_MFS_IMO == CapSenseProx_MFS_METHOD) */

                 /* Scan the next sensor */
                CapSenseProx_SsCSDInitNextScan();
            }
        #else
            /* Scan the next sensor */
            CapSenseProx_SsCSDInitNextScan();
        #endif /* (CapSenseProx_ENABLE == CapSenseProx_MULTI_FREQ_SCAN_EN) */

        #ifdef CapSenseProx_EXIT_CALLBACK
            CapSenseProx_ExitCallback();
        #endif /* CapSenseProx_EXIT_CALLBACK */
    }


    #if (CapSenseProx_ENABLE == CapSenseProx_CSD_GANGED_SNS_EN)
    /*******************************************************************************
    * Function Name: CapSenseProx_CSDPostMultiScanGanged
    ****************************************************************************//**
    *
    * \brief
    *  This is an internal ISR function for the multiple-sensor scanning
    *  implementation for ganged sensors.
    *
    * \details
    *  This ISR handler is triggered when the user calls the
    *  CapSenseProx_Scan() API for a ganged sensor or the
    *  CapSenseProx_ScanAllWidgets() API in the project with ganged sensors.
    *
    *  The following tasks are performed:
    *    1. Disable the CSD interrupt.
    *    2. Read the Counter register and update the data structure with raw data.
    *    3. Connect the Vref buffer to the AMUX bus.
    *    4. Disable the CSD block (after the widget has been scanned).
    *    5. Update the Scan Counter.
    *    6. Reset the BUSY flag.
    *    7. Enable the CSD interrupt.
    *
    *  The ISR handler initializes scanning for the previous sensor when the
    *  widget has more than one sensor.
    *  The ISR handler initializes scanning for the next widget when the
    *  CapSenseProx_ScanAllWidgets() APIs are called and the project has
    *  more than one widget.
    *  The ISR handler changes the IMO and initializes scanning for the next
    *  frequency channels when multi-frequency scanning is enabled.
    *
    *  This function has two Macro Callbacks that allow calling the user
    *  code from macros specified in Component's generated code. Refer to the
    *  \ref group_c_macrocallbacks section of the PSoC Creator User Guide
    *  for details.
    *
    *******************************************************************************/
    CY_ISR(CapSenseProx_CSDPostMultiScanGanged)
    {
        #ifdef CapSenseProx_ENTRY_CALLBACK
            CapSenseProx_EntryCallback();
        #endif /* CapSenseProx_ENTRY_CALLBACK */

        /* Clear pending interrupt */
        CY_SET_REG32(CapSenseProx_INTR_PTR, CapSenseProx_INTR_SET_MASK);

         /* Read Rawdata */
        CapSenseProx_SsCSDPostScan();

        /* Connect Vref Buffer to AMUX bus */
        CY_SET_REG32(CapSenseProx_CONFIG_PTR, CapSenseProx_configCsd | CapSenseProx_CTANK_PRECHARGE_CONFIG_CSD_EN);

        #if (CapSenseProx_ENABLE == CapSenseProx_MULTI_FREQ_SCAN_EN)
            if (CapSenseProx_FREQ_CHANNEL_2 == CapSenseProx_scanFreqIndex)
            {
                CapSenseProx_SsCSDDisconnectSnsExt((uint32)CapSenseProx_widgetIndex, (uint32)CapSenseProx_sensorIndex);
            }
        #else
            CapSenseProx_SsCSDDisconnectSnsExt((uint32)CapSenseProx_widgetIndex, (uint32)CapSenseProx_sensorIndex);
        #endif /* (CapSenseProx_ENABLE == CapSenseProx_MULTI_FREQ_SCAN_EN) */

        #if (CapSenseProx_ENABLE == CapSenseProx_MULTI_FREQ_SCAN_EN)
            if (CapSenseProx_FREQ_CHANNEL_2 > CapSenseProx_scanFreqIndex)
            {
                 /* Scan the next channel */
                CapSenseProx_SsNextFrequencyScan();
            }
            else
            {
                /* All channels are scanned. Set IMO to zero channel */
                CapSenseProx_scanFreqIndex = CapSenseProx_FREQ_CHANNEL_0;

                #if (CapSenseProx_MFS_IMO == CapSenseProx_MFS_METHOD)
                    CapSenseProx_SsChangeImoFreq(CapSenseProx_FREQ_CHANNEL_0);
                #else
                    CapSenseProx_SsChangeClkFreq(CapSenseProx_FREQ_CHANNEL_0);
                #endif /* (CapSenseProx_MFS_IMO == CapSenseProx_MFS_METHOD) */

                 /* Scan the next sensor */
                CapSenseProx_SsCSDInitNextScan();
            }
        #else
             /* Scan the next sensor */
            CapSenseProx_SsCSDInitNextScan();
        #endif /* (CapSenseProx_ENABLE == CapSenseProx_MULTI_FREQ_SCAN_EN) */

        #ifdef CapSenseProx_EXIT_CALLBACK
            CapSenseProx_ExitCallback();
        #endif /* CapSenseProx_EXIT_CALLBACK */
    }
    #endif /* (CapSenseProx_ENABLE == CapSenseProx_CSD_GANGED_SNS_EN) */

#endif /* (CapSenseProx_ENABLE == CapSenseProx_CSDV2) */

#endif /* ((CapSenseProx_ENABLE == CapSenseProx_CSD_EN) || (CapSenseProx_ENABLE == CapSenseProx_CSD_CSX_EN)) */

/** \}
 * \endcond */


#if ((CapSenseProx_ENABLE == CapSenseProx_CSD_EN) || (CapSenseProx_ENABLE == CapSenseProx_CSD_CSX_EN))

/*******************************************************************************
* Function Name: CapSenseProx_SsCSDPostScan
****************************************************************************//**
*
* \brief
*   This function reads rawdata and releases required HW resources after scan.
*
* \details
*   This function performs following tasks after scan:
*   - Reads SlotResult from Raw Counter;
*   - Inits bad Conversions number;
*   - Disconnects Vrefhi from AMUBUF positive input;
*   - Disconnects AMUBUF output from CSDBUSB with sych PHI2+HSCMP;
*   - Opens HCBV and HCBG switches.
*
*******************************************************************************/
static void CapSenseProx_SsCSDPostScan(void)
{
    #if (CapSenseProx_ENABLE == CapSenseProx_CSDV2)

        uint32 tmpRawData;
        uint32 tmpMaxCount;
        CapSenseProx_RAM_WD_BASE_STRUCT const *ptrWdgt = (CapSenseProx_RAM_WD_BASE_STRUCT *)
                                            CapSenseProx_dsFlash.wdgtArray[CapSenseProx_widgetIndex].ptr2WdgtRam;

        /* Read SlotResult from Raw Counter */
        tmpRawData = CapSenseProx_RESULT_VAL1_VALUE_MASK & CY_GET_REG32(CapSenseProx_COUNTER_PTR);

        tmpMaxCount = ((1uL << ptrWdgt->resolution) - 1uL);
        if(tmpRawData < tmpMaxCount)
        {
            CapSenseProx_curRamSnsPtr->raw[CapSenseProx_scanFreqIndex] = LO16(tmpRawData);
        }
        else
        {
            CapSenseProx_curRamSnsPtr->raw[CapSenseProx_scanFreqIndex] = LO16(tmpMaxCount);
        }

        #if (CapSenseProx_ENABLE == CapSenseProx_CSD_NOISE_METRIC_EN)
            /* Init bad Conversions number */
            CapSenseProx_badConversionsNum = CapSenseProx_BAD_CONVERSIONS_NUM;
        #endif /* (CapSenseProx_ENABLE == CapSenseProx_CSD_NOISE_METRIC_EN) */

        #if (CapSenseProx_ENABLE == CapSenseProx_CSD_SHIELD_EN)
            /* Open HCBV and HCBG switches */
            CY_SET_REG32(CapSenseProx_SW_SHIELD_SEL_PTR, CapSenseProx_SW_SHIELD_SEL_SW_HCBV_STATIC_OPEN |
                                                             CapSenseProx_SW_SHIELD_SEL_SW_HCBG_STATIC_OPEN);
        #endif /* (CapSenseProx_ENABLE == CapSenseProx_CSD_SHIELD_EN) */

    #else

        /* Read SlotResult from Raw Counter */
       CapSenseProx_curRamSnsPtr->raw[CapSenseProx_scanFreqIndex] = (uint16)CY_GET_REG32(CapSenseProx_COUNTER_PTR);

    #endif /* (CapSenseProx_ENABLE == CapSenseProx_CSDV2) */
}


/*******************************************************************************
* Function Name: CapSenseProx_SsCSDInitNextScan
****************************************************************************//**
*
* \brief
*   This function initializes the next sensor scan.
*
* \details
*   The function increments the sensor index, updates sense clock for matrix
*   or touchpad widgets only, sets up Compensation IDAC, enables the sensor and
*   scans it. When all the sensors are scanned it continues to set up the next widget
*   until all the widgets are scanned. The CSD block is disabled when all the widgets are
*   scanned.
*
*******************************************************************************/
static void CapSenseProx_SsCSDInitNextScan(void)
{
    /* Declare and initialize ptr to widget and sensor structures to appropriate address */
    #if (((CapSenseProx_ENABLE == CapSenseProx_CSD_IDAC_COMP_EN) || \
          (CapSenseProx_CSD_MATRIX_WIDGET_EN || CapSenseProx_CSD_TOUCHPAD_WIDGET_EN)) || \
          (((CapSenseProx_DISABLE == CapSenseProx_CSD_COMMON_SNS_CLK_EN) && \
          (CapSenseProx_ENABLE == CapSenseProx_CSDV2) && \
          (CapSenseProx_CSD_MATRIX_WIDGET_EN || CapSenseProx_CSD_TOUCHPAD_WIDGET_EN))))
        CapSenseProx_RAM_WD_BASE_STRUCT *ptrWdgt = (CapSenseProx_RAM_WD_BASE_STRUCT *)
                                                        CapSenseProx_dsFlash.wdgtArray[CapSenseProx_widgetIndex].ptr2WdgtRam;
    #endif

    /* Check if all the sensors are scanned in widget */
    if (((uint8)CapSenseProx_dsFlash.wdgtArray[(CapSenseProx_widgetIndex)].totalNumSns - 1u) > CapSenseProx_sensorIndex)
    {
        /* Increment sensor index to configure next sensor in widget */
        CapSenseProx_sensorIndex++;

        /* Update global pointer to CapSenseProx_RAM_SNS_STRUCT to current sensor  */
        CapSenseProx_curRamSnsPtr = (CapSenseProx_RAM_SNS_STRUCT *)
                                                  CapSenseProx_dsFlash.wdgtArray[CapSenseProx_widgetIndex].ptr2SnsRam
                                                  + CapSenseProx_sensorIndex;

        /* Configure clock divider to row or column */
        #if ((CapSenseProx_DISABLE == CapSenseProx_CSD_COMMON_SNS_CLK_EN) && \
             (CapSenseProx_CSD_MATRIX_WIDGET_EN || CapSenseProx_CSD_TOUCHPAD_WIDGET_EN))
            CapSenseProx_SsCSDConfigClock();

            #if (CapSenseProx_ENABLE == CapSenseProx_CSDV2)
                 /* Set up scanning resolution */
                CapSenseProx_SsCSDCalculateScanDuration(ptrWdgt);
            #endif
        #endif

        /* Setup Compensation IDAC for next sensor in widget */
        #if ((CapSenseProx_ENABLE == CapSenseProx_CSD_IDAC_COMP_EN) || \
             (CapSenseProx_CSD_MATRIX_WIDGET_EN || CapSenseProx_CSD_TOUCHPAD_WIDGET_EN))
            CapSenseProx_SsCSDSetUpIdacs(ptrWdgt);
        #endif /* ((CapSenseProx_ENABLE == CapSenseProx_CSD_IDAC_COMP_EN) || \
                   (CapSenseProx_CSD_MATRIX_WIDGET_EN || CapSenseProx_CSD_TOUCHPAD_WIDGET_EN)) */

        /* Enable sensor */
        CapSenseProx_SsCSDConnectSensorExt((uint32)CapSenseProx_widgetIndex, (uint32)CapSenseProx_sensorIndex);

        /* Proceed scanning */
        CapSenseProx_SsCSDStartSample();
    }
    /* Call scan next widget API if requested, if not, complete the scan  */
    else
    {
        CapSenseProx_sensorIndex = 0u;

        /* Current widget is totally scanned. Reset WIDGET BUSY flag */
        CapSenseProx_dsRam.status &= ~CapSenseProx_WDGT_SW_STS_BUSY;

        /* Check if all the widgets have been scanned */
        if (CapSenseProx_ENABLE == CapSenseProx_requestScanAllWidget)
        {
            /* Configure and begin scanning next widget */
            CapSenseProx_SsPostAllWidgetsScan();
        }
        else
        {
            #if (CapSenseProx_ENABLE == CapSenseProx_BLOCK_OFF_AFTER_SCAN_EN)
                #if (CapSenseProx_ENABLE == CapSenseProx_CSDV2)
                    /* Disable the CSD block */
                    CY_SET_REG32(CapSenseProx_CONFIG_PTR, CapSenseProx_configCsd);
                #else
                    /* Disable the CSD block. Connect Vref Buffer to AMUX bus */
                    #if ((CapSenseProx_CSH_PRECHARGE_IO_BUF == CapSenseProx_CSD_CSH_PRECHARGE_SRC) &&\
                         (CapSenseProx_ENABLE == CapSenseProx_CSD_SHIELD_TANK_EN))
                        CY_SET_REG32(CapSenseProx_CONFIG_PTR, CapSenseProx_configCsd | CapSenseProx_CMOD_PRECHARGE_CONFIG);
                    #else
                        CY_SET_REG32(CapSenseProx_CONFIG_PTR, CapSenseProx_configCsd | CapSenseProx_CTANK_PRECHARGE_CONFIG);
                    #endif
                #endif
            #endif

            /* All widgets are totally scanned. Reset BUSY flag */
            CapSenseProx_dsRam.status &= ~CapSenseProx_SW_STS_BUSY;

            /* Update scan Counter */
            CapSenseProx_dsRam.scanCounter++;
        }
    }
}

#if (CapSenseProx_ENABLE == CapSenseProx_MULTI_FREQ_SCAN_EN)
    /*******************************************************************************
    * Function Name: CapSenseProx_SsNextFrequencyScan
    ****************************************************************************//**
    *
    * \brief
    *   This function scans the sensor on the next frequency channel.
    *
    * \details
    *   The function increments the frequency channel, changes IMO and initializes
    *   the scanning process of the same sensor.
    *
    *******************************************************************************/
    static void CapSenseProx_SsNextFrequencyScan(void)
    {
        CapSenseProx_RAM_WD_BASE_STRUCT const *ptrWdgt = (CapSenseProx_RAM_WD_BASE_STRUCT *)
                                                        CapSenseProx_dsFlash.wdgtArray[CapSenseProx_widgetIndex].ptr2WdgtRam;

        CapSenseProx_scanFreqIndex++;

        /* Set Immunity */
        #if (CapSenseProx_MFS_IMO == CapSenseProx_MFS_METHOD)
            CapSenseProx_SsChangeImoFreq((uint32)CapSenseProx_scanFreqIndex);
        #else
            CapSenseProx_SsChangeClkFreq((uint32)CapSenseProx_scanFreqIndex);
        #endif /* (CapSenseProx_MFS_IMO == CapSenseProx_MFS_METHOD) */

        /* Update IDAC registers */
        CapSenseProx_SsCSDSetUpIdacs(ptrWdgt);

        /* Proceed scanning */
        CapSenseProx_SsCSDStartSample();
    }
#endif /* (CapSenseProx_ENABLE == CapSenseProx_MULTI_FREQ_SCAN_EN) */

#endif /* ((CapSenseProx_ENABLE == CapSenseProx_CSD_EN) || (CapSenseProx_ENABLE == CapSenseProx_CSD_CSX_EN)) */


/* [] END OF FILE */
