/***************************************************************************//**
* \file CapSensep_INT.c
* \version 7.0
*
* \brief
*   This file contains the source code for implementation of the Component's
*   Interrupt Service Routine (ISR).
*
* \see CapSensep v7.0 Datasheet
*
*//*****************************************************************************
* Copyright (2016-2019), Cypress Semiconductor Corporation.
********************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress) and is
* protected by and subject to worldwide patent protection (United States and
* foreign), United States copyright laws and international treaty provisions.
* Cypress hereby grants to licensee a personal, non-exclusive, non-transferable
* license to copy, use, modify, create derivative works of, and compile the
* Cypress Source Code and derivative works for the sole purpose of creating
* custom software in support of licensee product to be used only in conjunction
* with a Cypress integrated circuit as specified in the applicable agreement.
* Any reproduction, modification, translation, compilation, or representation of
* this software except as specified above is prohibited without the express
* written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the
* materials described herein. Cypress does not assume any liability arising out
* of the application or use of any product or circuit described herein. Cypress
* does not authorize its products for use as critical components in life-support
* systems where a malfunction or failure may reasonably be expected to result in
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of
* such use and in doing so indemnifies Cypress against all charges. Use may be
* limited by and subject to the applicable Cypress software license agreement.
*******************************************************************************/
#include "cytypes.h"
#include "cyfitter.h"
#include "CapSensep_Configuration.h"
#include "CapSensep_Structure.h"
#include "CapSensep_Sensing.h"
#if (CapSensep_ENABLE == CapSensep_CSD_EN)
    #include "CapSensep_SensingCSD_LL.h"
#endif /* (CapSensep_ENABLE == CapSensep_CSD_EN) */
#include "cyapicallbacks.h"

/*******************************************************************************
* Static Function Prototypes
*******************************************************************************/
/**
* \cond SECTION_C_INTERNAL
* \addtogroup group_c_internal
* \{
*/

#if (((CapSensep_ENABLE == CapSensep_CSD_EN) || (CapSensep_ENABLE == CapSensep_CSD_CSX_EN)) && \
     (CapSensep_ENABLE == CapSensep_MULTI_FREQ_SCAN_EN))
    static void CapSensep_SsNextFrequencyScan(void);
#endif /* (((CapSensep_ENABLE == CapSensep_CSD_EN) || (CapSensep_ENABLE == CapSensep_CSD_CSX_EN)) && \
            (CapSensep_ENABLE == CapSensep_MULTI_FREQ_SCAN_EN)) */

#if ((CapSensep_ENABLE == CapSensep_CSD_EN) || (CapSensep_ENABLE == CapSensep_CSD_CSX_EN))
    static void CapSensep_SsCSDPostScan(void);
    static void CapSensep_SsCSDInitNextScan(void);
#endif /* ((CapSensep_ENABLE == CapSensep_CSD_EN) || (CapSensep_ENABLE == CapSensep_CSD_CSX_EN)) */
/** \}
* \endcond */


/**
* \cond SECTION_C_INTERRUPT
* \addtogroup group_c_interrupt
* \{
*/


#if ((CapSensep_ENABLE == CapSensep_CSD_EN) || (CapSensep_ENABLE == CapSensep_CSD_CSX_EN))

#if (CapSensep_ENABLE == CapSensep_CSDV2)
    /* Fourth-generation HW block part */

    /*******************************************************************************
    * Function Name: CapSensep_CSDPostSingleScan
    ****************************************************************************//**
    *
    * \brief
    *  This is an internal ISR function for the single-sensor scanning implementation.
    *
    * \details
    *  This ISR handler is triggered when the user calls the
    *  CapSensep_CSDScanExt() function.
    *
    *  The following tasks are performed for Third-generation HW block:
    *    1. Disable the CSD interrupt.
    *    2. Read the Counter register and update the data structure with raw data.
    *    3. Connect the Vref buffer to the AMUX bus.
    *    4. Update the Scan Counter.
    *    5. Reset the BUSY flag.
    *    6. Enable the CSD interrupt.
    *
    *  The following tasks are performed for Fourth-generation HW block:
    *    1. Check if the raw data is not noisy.
    *    2. Read the Counter register and update the data structure with raw data.
    *    3. Configure and start the scan for the next frequency if the
    *      multi-frequency is enabled.
    *    4. Update the Scan Counter.
    *    5. Reset the BUSY flag.
    *    6. Enable the CSD interrupt.
    *
    *  The ISR handler changes the IMO and initializes scanning for the next frequency
    *  channels when multi-frequency scanning is enabled.
    *
    *  This function has two Macro Callbacks that allow calling the user code
    *  from macros specified in Component's generated code. Refer to the
    *  \ref group_c_macrocallbacks section of the PSoC Creator User Guide
    *  for details.
    *
    *******************************************************************************/
    CY_ISR(CapSensep_CSDPostSingleScan)
    {
        #ifdef CapSensep_ENTRY_CALLBACK
            CapSensep_EntryCallback();
        #endif /* CapSensep_ENTRY_CALLBACK */

        /* Clear pending interrupt */
        CY_SET_REG32(CapSensep_INTR_PTR, CapSensep_INTR_SET_MASK);

        #if (CapSensep_ENABLE == CapSensep_CSD_NOISE_METRIC_EN)
            if ((CapSensep_CSD_NOISE_METRIC_TH < ((CY_GET_REG32(CapSensep_RESULT_VAL1_PTR) &
                                                        CapSensep_RESULT_VAL1_BAD_CONVS_MASK) >>
                                                        CapSensep_RESULT_VAL1_BAD_CONVS_SHIFT)) &&
                                                        (0u < CapSensep_badConversionsNum))
            {
                /* Decrement bad conversions number */
                CapSensep_badConversionsNum--;

                /* Start the re-scan */
                CY_SET_REG32(CapSensep_SEQ_START_PTR, CapSensep_SEQ_START_AZ0_SKIP_MASK |
                                                             CapSensep_SEQ_START_AZ1_SKIP_MASK |
                                                             CapSensep_SEQ_START_START_MASK);
            }
            else
            {
        #endif /* (CapSensep_ENABLE == CapSensep_CSD_NOISE_METRIC_EN) */

            CapSensep_SsCSDPostScan();

            #if (CapSensep_ENABLE == CapSensep_MULTI_FREQ_SCAN_EN)
                if (CapSensep_FREQ_CHANNEL_2 > CapSensep_scanFreqIndex)
                {
                    /* Scan the next channel */
                    CapSensep_SsNextFrequencyScan();
                }
                else
                {
                    /* All channels are scanned. Set IMO to zero channel */
                    CapSensep_scanFreqIndex = CapSensep_FREQ_CHANNEL_0;

                    #if (CapSensep_MFS_IMO == CapSensep_MFS_METHOD)
                        CapSensep_SsChangeImoFreq(CapSensep_FREQ_CHANNEL_0);
                    #else
                        CapSensep_SsChangeClkFreq(CapSensep_FREQ_CHANNEL_0);
                    #endif /* (CapSensep_MFS_IMO == CapSensep_MFS_METHOD) */

                    #if (CapSensep_ENABLE == CapSensep_BLOCK_OFF_AFTER_SCAN_EN)
                        /* Disable Fourth-generation HW block */
                        CY_SET_REG32(CapSensep_CONFIG_PTR, CapSensep_configCsd);
                    #endif /* (CapSensep_ENABLE == CapSensep_BLOCK_OFF_AFTER_SCAN_EN) */

                    /* Update Scan Counter */
                    CapSensep_dsRam.scanCounter++;

                    /* Sensor is totally scanned. Reset BUSY flag */
                    CapSensep_dsRam.status &= ~(CapSensep_SW_STS_BUSY | CapSensep_WDGT_SW_STS_BUSY);
                }
            #else
                {
                    #if (CapSensep_ENABLE == CapSensep_BLOCK_OFF_AFTER_SCAN_EN)
                        /* Disable Fourth-generation HW block */
                        CY_SET_REG32(CapSensep_CONFIG_PTR, CapSensep_configCsd);
                    #endif /* (CapSensep_ENABLE == CapSensep_BLOCK_OFF_AFTER_SCAN_EN) */

                    /* Update Scan Counter */
                    CapSensep_dsRam.scanCounter++;

                    /* Sensor is totally scanned. Reset BUSY flag */
                    CapSensep_dsRam.status &= ~(CapSensep_SW_STS_BUSY | CapSensep_WDGT_SW_STS_BUSY);
                }
            #endif /* (CapSensep_ENABLE == CapSensep_MULTI_FREQ_SCAN_EN) */
    #if (CapSensep_ENABLE == CapSensep_CSD_NOISE_METRIC_EN)
        }
    #endif /* (CapSensep_ENABLE == CapSensep_CSD_NOISE_METRIC_EN) */

        #ifdef CapSensep_EXIT_CALLBACK
            CapSensep_ExitCallback();
        #endif /* CapSensep_EXIT_CALLBACK */
    }


    /*******************************************************************************
    * Function Name: CapSensep_CSDPostMultiScan
    ****************************************************************************//**
    *
    * \brief
    *  This is an internal ISR function for the multiple-sensor scanning
    *  implementation.
    *
    * \details
    *  This ISR handler is triggered when the user calls the
    *  CapSensep_Scan() or CapSensep_ScanAllWidgets() APIs.
    *
    *  The following tasks are performed:
    *    1. Disable the CSD interrupt.
    *    2. Read the Counter register and update the data structure with raw data.
    *    3. Connect the Vref buffer to the AMUX bus.
    *    4. Disable the CSD block (after the widget has been scanned).
    *    5. Update the Scan Counter.
    *    6. Reset the BUSY flag.
    *    7. Enable the CSD interrupt.
    *
    *  The ISR handler initializes scanning for the previous sensor when the
    *  widget has more than one sensor.
    *  The ISR handler initializes scanning for the next widget when the
    *  CapSensep_ScanAllWidgets() APIs are called and the project has
    *  more than one widget.
    *  The ISR handler changes the IMO and initializes scanning for the next
    *  frequency channels when multi-frequency scanning is enabled.
    *
    *  This function has two Macro Callbacks that allow calling the user
    *  code from macros specified in Component's generated code. Refer to the
    *  \ref group_c_macrocallbacks section of the PSoC Creator User Guide
    *  for details.
    *
    *******************************************************************************/
    CY_ISR(CapSensep_CSDPostMultiScan)
    {
        /* Declare and initialize ptr to sensor IO structure to appropriate address */
        CapSensep_FLASH_IO_STRUCT const *curSnsIOPtr = (CapSensep_FLASH_IO_STRUCT const *)
                                                          CapSensep_dsFlash.wdgtArray[CapSensep_widgetIndex].ptr2SnsFlash
                                                          + CapSensep_sensorIndex;

        #ifdef CapSensep_ENTRY_CALLBACK
            CapSensep_EntryCallback();
        #endif /* CapSensep_ENTRY_CALLBACK */

        /* Clear pending interrupt */
        CY_SET_REG32(CapSensep_INTR_PTR, CapSensep_INTR_SET_MASK);

        #if (CapSensep_ENABLE == CapSensep_CSD_NOISE_METRIC_EN)
            if ((CapSensep_CSD_NOISE_METRIC_TH < ((CY_GET_REG32(CapSensep_RESULT_VAL1_PTR) &
                                                      CapSensep_RESULT_VAL1_BAD_CONVS_MASK) >>
                                                      CapSensep_RESULT_VAL1_BAD_CONVS_SHIFT)) &&
                                                      (0u < CapSensep_badConversionsNum))
            {
                /* Decrement bad conversions number */
                CapSensep_badConversionsNum--;

                /* Start the re-scan */
                CY_SET_REG32(CapSensep_SEQ_START_PTR, CapSensep_SEQ_START_AZ0_SKIP_MASK |
                                                             CapSensep_SEQ_START_AZ1_SKIP_MASK |
                                                             CapSensep_SEQ_START_START_MASK);
            }
            else
            {
        #endif /* (CapSensep_ENABLE == CapSensep_CSD_NOISE_METRIC_EN) */

            CapSensep_SsCSDPostScan();

            /* Disable sensor when all frequency channels are scanned */
            #if (CapSensep_ENABLE == CapSensep_MULTI_FREQ_SCAN_EN)
                if (CapSensep_FREQ_CHANNEL_2 == CapSensep_scanFreqIndex)
            #endif /* (CapSensep_ENABLE == CapSensep_MULTI_FREQ_SCAN_EN) */
            {
                /* Disable sensor */
                CapSensep_CSDDisconnectSns(curSnsIOPtr);
            }

            #if (CapSensep_ENABLE == CapSensep_MULTI_FREQ_SCAN_EN)
                if (CapSensep_FREQ_CHANNEL_2 > CapSensep_scanFreqIndex)
                {
                     /* Scan the next channel */
                    CapSensep_SsNextFrequencyScan();
                }
                else
                {
                     /* All channels are scanned. Set IMO to zero channel */
                    CapSensep_scanFreqIndex = CapSensep_FREQ_CHANNEL_0;

                    #if (CapSensep_MFS_IMO == CapSensep_MFS_METHOD)
                        CapSensep_SsChangeImoFreq(CapSensep_FREQ_CHANNEL_0);
                    #else
                        CapSensep_SsChangeClkFreq(CapSensep_FREQ_CHANNEL_0);
                    #endif /* (CapSensep_MFS_IMO == CapSensep_MFS_METHOD) */

                     /* Scan the next sensor */
                    CapSensep_SsCSDInitNextScan();
                }
            #else
                /* Scan the next sensor */
                CapSensep_SsCSDInitNextScan();
            #endif /* (CapSensep_ENABLE == CapSensep_MULTI_FREQ_SCAN_EN) */
        #if (CapSensep_ENABLE == CapSensep_CSD_NOISE_METRIC_EN)
            }
        #endif /* (CapSensep_ENABLE == CapSensep_CSD_NOISE_METRIC_EN) */

        #ifdef CapSensep_EXIT_CALLBACK
            CapSensep_ExitCallback();
        #endif /* CapSensep_EXIT_CALLBACK */
    }


    #if (CapSensep_ENABLE == CapSensep_CSD_GANGED_SNS_EN)
    /*******************************************************************************
    * Function Name: CapSensep_CSDPostMultiScanGanged
    ****************************************************************************//**
    *
    * \brief
    *  This is an internal ISR function for the multiple-sensor scanning
    *  implementation for ganged sensors.
    *
    * \details
    *  This ISR handler is triggered when the user calls the
    *  CapSensep_Scan() API for a ganged sensor or the
    *  CapSensep_ScanAllWidgets() API in the project with ganged sensors.
    *
    *  The following tasks are performed:
    *    1. Disable the CSD interrupt.
    *    2. Read the Counter register and update the data structure with raw data.
    *    3. Connect the Vref buffer to the AMUX bus.
    *    4. Disable the CSD block (after the widget has been scanned).
    *    5. Update the Scan Counter.
    *    6. Reset the BUSY flag.
    *    7. Enable the CSD interrupt.
    *
    *  The ISR handler initializes scanning for the previous sensor when the
    *  widget has more than one sensor.
    *  The ISR handler initializes scanning for the next widget when the
    *  CapSensep_ScanAllWidgets() APIs are called and the project has
    *  more than one widget.
    *  The ISR handler changes the IMO and initializes scanning for the next
    *  frequency channels when multi-frequency scanning is enabled.
    *
    *  This function has two Macro Callbacks that allow calling the user
    *  code from macros specified in Component's generated code. Refer to the
    *  \ref group_c_macrocallbacks section of the PSoC Creator User Guide
    *  for details.
    *
    *******************************************************************************/
    CY_ISR(CapSensep_CSDPostMultiScanGanged)
    {
        #ifdef CapSensep_ENTRY_CALLBACK
            CapSensep_EntryCallback();
        #endif /* CapSensep_ENTRY_CALLBACK */

        /* Clear pending interrupt */
        CY_SET_REG32(CapSensep_INTR_PTR, CapSensep_INTR_SET_MASK);

        #if (CapSensep_ENABLE == CapSensep_CSD_NOISE_METRIC_EN)
            if ((CapSensep_CSD_NOISE_METRIC_TH < ((CY_GET_REG32(CapSensep_RESULT_VAL1_PTR) &
                                                      CapSensep_RESULT_VAL1_BAD_CONVS_MASK) >>
                                                      CapSensep_RESULT_VAL1_BAD_CONVS_SHIFT)) &&
                                                      (0u < CapSensep_badConversionsNum))
            {
                /* Decrement bad conversions number */
                CapSensep_badConversionsNum--;

                /* Start the re-scan */
                CY_SET_REG32(CapSensep_SEQ_START_PTR, CapSensep_SEQ_START_AZ0_SKIP_MASK |
                                                             CapSensep_SEQ_START_AZ1_SKIP_MASK |
                                                             CapSensep_SEQ_START_START_MASK);
            }
            else
            {
        #endif /* (CapSensep_ENABLE == CapSensep_CSD_NOISE_METRIC_EN) */

            CapSensep_SsCSDPostScan();

            #if (CapSensep_ENABLE == CapSensep_MULTI_FREQ_SCAN_EN)
                if (CapSensep_FREQ_CHANNEL_2 == CapSensep_scanFreqIndex)
            #endif /* (CapSensep_ENABLE == CapSensep_MULTI_FREQ_SCAN_EN) */
            {
                CapSensep_SsCSDDisconnectSnsExt((uint32)CapSensep_widgetIndex, (uint32)CapSensep_sensorIndex);
            }

            #if (CapSensep_ENABLE == CapSensep_MULTI_FREQ_SCAN_EN)
                if (CapSensep_FREQ_CHANNEL_2 > CapSensep_scanFreqIndex)
                {
                     /* Scan the next channel */
                    CapSensep_SsNextFrequencyScan();
                }
                else
                {
                    /* All channels are scanned. Set IMO to zero channel */
                    CapSensep_scanFreqIndex = CapSensep_FREQ_CHANNEL_0;

                    #if (CapSensep_MFS_IMO == CapSensep_MFS_METHOD)
                        CapSensep_SsChangeImoFreq(CapSensep_FREQ_CHANNEL_0);
                    #else
                        CapSensep_SsChangeClkFreq(CapSensep_FREQ_CHANNEL_0);
                    #endif /* (CapSensep_MFS_IMO == CapSensep_MFS_METHOD) */

                     /* Scan the next sensor */
                    CapSensep_SsCSDInitNextScan();
                }
            #else
                 /* Scan the next sensor */
                CapSensep_SsCSDInitNextScan();
            #endif /* (CapSensep_ENABLE == CapSensep_MULTI_FREQ_SCAN_EN) */
        #if (CapSensep_ENABLE == CapSensep_CSD_NOISE_METRIC_EN)
            }
        #endif /* (CapSensep_ENABLE == CapSensep_CSD_NOISE_METRIC_EN) */

        #ifdef CapSensep_EXIT_CALLBACK
            CapSensep_ExitCallback();
        #endif /* CapSensep_EXIT_CALLBACK */
    }
    #endif /* (CapSensep_ENABLE == CapSensep_CSD_GANGED_SNS_EN) */

#else

    /* Third-generation HW block part */

    /*******************************************************************************
    * Function Name: CapSensep_CSDPostSingleScan
    ****************************************************************************//**
    *
    * \brief
    *  This is an internal ISR function for the single-sensor scanning implementation.
    *
    * \details
    *  This ISR handler is triggered when the user calls the
    *  CapSensep_CSDScanExt() function.
    *
    *  The following tasks are performed for Third-generation HW block:
    *    1. Disable the CSD interrupt.
    *    2. Read the Counter register and update the data structure with raw data.
    *    3. Connect the Vref buffer to the AMUX bus.
    *    4. Update the Scan Counter.
    *    5. Reset the BUSY flag.
    *    6. Enable the CSD interrupt.
    *
    *  The following tasks are performed for Fourth-generation HW block:
    *    1. Check if the raw data is not noisy.
    *    2. Read the Counter register and update the data structure with raw data.
    *    3. Configure and start the scan for the next frequency if the
    *      multi-frequency is enabled.
    *    4. Update the Scan Counter.
    *    5. Reset the BUSY flag.
    *    6. Enable the CSD interrupt.
    *
    *  The ISR handler changes the IMO and initializes scanning for the next frequency
    *  channels when multi-frequency scanning is enabled.
    *
    *  This function has two Macro Callbacks that allow calling the user code
    *  from macros specified in Component's generated code. Refer to the
    *  \ref group_c_macrocallbacks section of the PSoC Creator User Guide
    *  for details.
    *
    *******************************************************************************/
    CY_ISR(CapSensep_CSDPostSingleScan)
    {
        #ifdef CapSensep_ENTRY_CALLBACK
            CapSensep_EntryCallback();
        #endif /* CapSensep_ENTRY_CALLBACK */

        /* Clear pending interrupt */
        CY_SET_REG32(CapSensep_INTR_PTR, CapSensep_INTR_SET_MASK);

        /* Read Rawdata */
        CapSensep_SsCSDPostScan();

        #if (CapSensep_ENABLE == CapSensep_MULTI_FREQ_SCAN_EN)
            if (CapSensep_FREQ_CHANNEL_2 > CapSensep_scanFreqIndex)
            {
                /* Connect Vref Buffer to AMUX bus. Third-generation HW block is enabled */
                CY_SET_REG32(CapSensep_CONFIG_PTR, CapSensep_configCsd | CapSensep_CTANK_PRECHARGE_CONFIG_CSD_EN);

                CapSensep_SsNextFrequencyScan();
            }
            else
            {
                CapSensep_scanFreqIndex = CapSensep_FREQ_CHANNEL_0;

                #if (CapSensep_MFS_IMO == CapSensep_MFS_METHOD)
                    CapSensep_SsChangeImoFreq(CapSensep_FREQ_CHANNEL_0);
                #else
                    CapSensep_SsChangeClkFreq(CapSensep_FREQ_CHANNEL_0);
                #endif /* (CapSensep_MFS_IMO == CapSensep_MFS_METHOD) */

                #if (CapSensep_ENABLE == CapSensep_BLOCK_OFF_AFTER_SCAN_EN)
                    /* Disable Third-generation HW block. Connect Vref Buffer to AMUX bus */
                    #if ((CapSensep_CSH_PRECHARGE_IO_BUF == CapSensep_CSD_CSH_PRECHARGE_SRC) &&\
                         (CapSensep_ENABLE == CapSensep_CSD_SHIELD_TANK_EN))
                        CY_SET_REG32(CapSensep_CONFIG_PTR, CapSensep_configCsd | CapSensep_CMOD_PRECHARGE_CONFIG);
                    #else
                        CY_SET_REG32(CapSensep_CONFIG_PTR, CapSensep_configCsd | CapSensep_CTANK_PRECHARGE_CONFIG);
                    #endif
                #else
                    /* Connect Vref Buffer to AMUX bus. Third-generation HW block is enabled */
                    CY_SET_REG32(CapSensep_CONFIG_PTR, CapSensep_configCsd | CapSensep_CTANK_PRECHARGE_CONFIG_CSD_EN);
                #endif

                /* Update Scan Counter */
                CapSensep_dsRam.scanCounter++;

                /* Sensor is totally scanned. Reset BUSY flag */
                CapSensep_dsRam.status &= ~(CapSensep_SW_STS_BUSY | CapSensep_WDGT_SW_STS_BUSY);
            }
        #else
            {
                #if (CapSensep_ENABLE == CapSensep_BLOCK_OFF_AFTER_SCAN_EN)
                    /* Disable Third-generation HW block. Connect Vref Buffer to AMUX bus */
                    #if ((CapSensep_CSH_PRECHARGE_IO_BUF == CapSensep_CSD_CSH_PRECHARGE_SRC) &&\
                         (CapSensep_ENABLE == CapSensep_CSD_SHIELD_TANK_EN))
                        CY_SET_REG32(CapSensep_CONFIG_PTR, CapSensep_configCsd | CapSensep_CMOD_PRECHARGE_CONFIG);
                    #else
                        CY_SET_REG32(CapSensep_CONFIG_PTR, CapSensep_configCsd | CapSensep_CTANK_PRECHARGE_CONFIG);
                    #endif
                #else
                    /* Connect Vref Buffer to AMUX bus. Third-generation HW block is enabled */
                    CY_SET_REG32(CapSensep_CONFIG_PTR, CapSensep_configCsd | CapSensep_CTANK_PRECHARGE_CONFIG_CSD_EN);
                #endif

                /* Update Scan Counter */
                CapSensep_dsRam.scanCounter++;

                /* Sensor is totally scanned. Reset BUSY flag */
                CapSensep_dsRam.status &= ~(CapSensep_SW_STS_BUSY | CapSensep_WDGT_SW_STS_BUSY);
            }
        #endif /* (CapSensep_ENABLE == CapSensep_MULTI_FREQ_SCAN_EN) */

        #ifdef CapSensep_EXIT_CALLBACK
            CapSensep_ExitCallback();
        #endif /* CapSensep_EXIT_CALLBACK */
    }


    /*******************************************************************************
    * Function Name: CapSensep_CSDPostMultiScan
    ****************************************************************************//**
    *
    * \brief
    *  This is an internal ISR function for the multiple-sensor scanning
    *  implementation.
    *
    * \details
    *  This ISR handler is triggered when the user calls the
    *  CapSensep_Scan() or CapSensep_ScanAllWidgets() APIs.
    *
    *  The following tasks are performed:
    *    1. Disable the CSD interrupt.
    *    2. Read the Counter register and update the data structure with raw data.
    *    3. Connect the Vref buffer to the AMUX bus.
    *    4. Disable the CSD block (after the widget has been scanned).
    *    5. Update the Scan Counter.
    *    6. Reset the BUSY flag.
    *    7. Enable the CSD interrupt.
    *
    *  The ISR handler initializes scanning for the previous sensor when the
    *  widget has more than one sensor.
    *  The ISR handler initializes scanning for the next widget when the
    *  CapSensep_ScanAllWidgets() APIs are called and the project has
    *  more than one widget.
    *  The ISR handler changes the IMO and initializes scanning for the next
    *  frequency channels when multi-frequency scanning is enabled.
    *
    *  This function has two Macro Callbacks that allow calling the user
    *  code from macros specified in Component's generated code. Refer to the
    *  \ref group_c_macrocallbacks section of the PSoC Creator User Guide
    *  for details.
    *
    *******************************************************************************/
    CY_ISR(CapSensep_CSDPostMultiScan)
    {
        /* Declare and initialize ptr to sensor IO structure to appropriate address        */
        CapSensep_FLASH_IO_STRUCT const *curSnsIOPtr = (CapSensep_FLASH_IO_STRUCT const *)
                                                          CapSensep_dsFlash.wdgtArray[CapSensep_widgetIndex].ptr2SnsFlash
                                                          + CapSensep_sensorIndex;

        #ifdef CapSensep_ENTRY_CALLBACK
            CapSensep_EntryCallback();
        #endif /* CapSensep_ENTRY_CALLBACK */

        /* Clear pending interrupt */
        CY_SET_REG32(CapSensep_INTR_PTR, CapSensep_INTR_SET_MASK);

         /* Read Rawdata */
        CapSensep_SsCSDPostScan();

        /* Connect Vref Buffer to AMUX bus */
        CY_SET_REG32(CapSensep_CONFIG_PTR, CapSensep_configCsd | CapSensep_CTANK_PRECHARGE_CONFIG_CSD_EN);

        #if (CapSensep_ENABLE == CapSensep_MULTI_FREQ_SCAN_EN)
            /* Disable sensor when all frequency channels are scanned */
            if (CapSensep_FREQ_CHANNEL_2 == CapSensep_scanFreqIndex)
            {
                /* Disable sensor */
                CapSensep_CSDDisconnectSns(curSnsIOPtr);
            }
        #else
            /* Disable sensor */
            CapSensep_CSDDisconnectSns(curSnsIOPtr);
        #endif /* (CapSensep_ENABLE == CapSensep_MULTI_FREQ_SCAN_EN) */

        #if (CapSensep_ENABLE == CapSensep_MULTI_FREQ_SCAN_EN)
            if (CapSensep_FREQ_CHANNEL_2 > CapSensep_scanFreqIndex)
            {
                 /* Scan the next channel */
                CapSensep_SsNextFrequencyScan();
            }
            else
            {
                 /* All channels are scanned. Set IMO to zero channel */
                CapSensep_scanFreqIndex = CapSensep_FREQ_CHANNEL_0;

                #if (CapSensep_MFS_IMO == CapSensep_MFS_METHOD)
                    CapSensep_SsChangeImoFreq(CapSensep_FREQ_CHANNEL_0);
                #else
                    CapSensep_SsChangeClkFreq(CapSensep_FREQ_CHANNEL_0);
                #endif /* (CapSensep_MFS_IMO == CapSensep_MFS_METHOD) */

                 /* Scan the next sensor */
                CapSensep_SsCSDInitNextScan();
            }
        #else
            /* Scan the next sensor */
            CapSensep_SsCSDInitNextScan();
        #endif /* (CapSensep_ENABLE == CapSensep_MULTI_FREQ_SCAN_EN) */

        #ifdef CapSensep_EXIT_CALLBACK
            CapSensep_ExitCallback();
        #endif /* CapSensep_EXIT_CALLBACK */
    }


    #if (CapSensep_ENABLE == CapSensep_CSD_GANGED_SNS_EN)
    /*******************************************************************************
    * Function Name: CapSensep_CSDPostMultiScanGanged
    ****************************************************************************//**
    *
    * \brief
    *  This is an internal ISR function for the multiple-sensor scanning
    *  implementation for ganged sensors.
    *
    * \details
    *  This ISR handler is triggered when the user calls the
    *  CapSensep_Scan() API for a ganged sensor or the
    *  CapSensep_ScanAllWidgets() API in the project with ganged sensors.
    *
    *  The following tasks are performed:
    *    1. Disable the CSD interrupt.
    *    2. Read the Counter register and update the data structure with raw data.
    *    3. Connect the Vref buffer to the AMUX bus.
    *    4. Disable the CSD block (after the widget has been scanned).
    *    5. Update the Scan Counter.
    *    6. Reset the BUSY flag.
    *    7. Enable the CSD interrupt.
    *
    *  The ISR handler initializes scanning for the previous sensor when the
    *  widget has more than one sensor.
    *  The ISR handler initializes scanning for the next widget when the
    *  CapSensep_ScanAllWidgets() APIs are called and the project has
    *  more than one widget.
    *  The ISR handler changes the IMO and initializes scanning for the next
    *  frequency channels when multi-frequency scanning is enabled.
    *
    *  This function has two Macro Callbacks that allow calling the user
    *  code from macros specified in Component's generated code. Refer to the
    *  \ref group_c_macrocallbacks section of the PSoC Creator User Guide
    *  for details.
    *
    *******************************************************************************/
    CY_ISR(CapSensep_CSDPostMultiScanGanged)
    {
        #ifdef CapSensep_ENTRY_CALLBACK
            CapSensep_EntryCallback();
        #endif /* CapSensep_ENTRY_CALLBACK */

        /* Clear pending interrupt */
        CY_SET_REG32(CapSensep_INTR_PTR, CapSensep_INTR_SET_MASK);

         /* Read Rawdata */
        CapSensep_SsCSDPostScan();

        /* Connect Vref Buffer to AMUX bus */
        CY_SET_REG32(CapSensep_CONFIG_PTR, CapSensep_configCsd | CapSensep_CTANK_PRECHARGE_CONFIG_CSD_EN);

        #if (CapSensep_ENABLE == CapSensep_MULTI_FREQ_SCAN_EN)
            if (CapSensep_FREQ_CHANNEL_2 == CapSensep_scanFreqIndex)
            {
                CapSensep_SsCSDDisconnectSnsExt((uint32)CapSensep_widgetIndex, (uint32)CapSensep_sensorIndex);
            }
        #else
            CapSensep_SsCSDDisconnectSnsExt((uint32)CapSensep_widgetIndex, (uint32)CapSensep_sensorIndex);
        #endif /* (CapSensep_ENABLE == CapSensep_MULTI_FREQ_SCAN_EN) */

        #if (CapSensep_ENABLE == CapSensep_MULTI_FREQ_SCAN_EN)
            if (CapSensep_FREQ_CHANNEL_2 > CapSensep_scanFreqIndex)
            {
                 /* Scan the next channel */
                CapSensep_SsNextFrequencyScan();
            }
            else
            {
                /* All channels are scanned. Set IMO to zero channel */
                CapSensep_scanFreqIndex = CapSensep_FREQ_CHANNEL_0;

                #if (CapSensep_MFS_IMO == CapSensep_MFS_METHOD)
                    CapSensep_SsChangeImoFreq(CapSensep_FREQ_CHANNEL_0);
                #else
                    CapSensep_SsChangeClkFreq(CapSensep_FREQ_CHANNEL_0);
                #endif /* (CapSensep_MFS_IMO == CapSensep_MFS_METHOD) */

                 /* Scan the next sensor */
                CapSensep_SsCSDInitNextScan();
            }
        #else
             /* Scan the next sensor */
            CapSensep_SsCSDInitNextScan();
        #endif /* (CapSensep_ENABLE == CapSensep_MULTI_FREQ_SCAN_EN) */

        #ifdef CapSensep_EXIT_CALLBACK
            CapSensep_ExitCallback();
        #endif /* CapSensep_EXIT_CALLBACK */
    }
    #endif /* (CapSensep_ENABLE == CapSensep_CSD_GANGED_SNS_EN) */

#endif /* (CapSensep_ENABLE == CapSensep_CSDV2) */

#endif /* ((CapSensep_ENABLE == CapSensep_CSD_EN) || (CapSensep_ENABLE == CapSensep_CSD_CSX_EN)) */

/** \}
 * \endcond */


#if ((CapSensep_ENABLE == CapSensep_CSD_EN) || (CapSensep_ENABLE == CapSensep_CSD_CSX_EN))

/*******************************************************************************
* Function Name: CapSensep_SsCSDPostScan
****************************************************************************//**
*
* \brief
*   This function reads rawdata and releases required HW resources after scan.
*
* \details
*   This function performs following tasks after scan:
*   - Reads SlotResult from Raw Counter;
*   - Inits bad Conversions number;
*   - Disconnects Vrefhi from AMUBUF positive input;
*   - Disconnects AMUBUF output from CSDBUSB with sych PHI2+HSCMP;
*   - Opens HCBV and HCBG switches.
*
*******************************************************************************/
static void CapSensep_SsCSDPostScan(void)
{
    #if (CapSensep_ENABLE == CapSensep_CSDV2)

        uint32 tmpRawData;
        uint32 tmpMaxCount;
        CapSensep_RAM_WD_BASE_STRUCT const *ptrWdgt = (CapSensep_RAM_WD_BASE_STRUCT *)
                                            CapSensep_dsFlash.wdgtArray[CapSensep_widgetIndex].ptr2WdgtRam;

        /* Read SlotResult from Raw Counter */
        tmpRawData = CapSensep_RESULT_VAL1_VALUE_MASK & CY_GET_REG32(CapSensep_COUNTER_PTR);

        tmpMaxCount = ((1uL << ptrWdgt->resolution) - 1uL);
        if(tmpRawData < tmpMaxCount)
        {
            CapSensep_curRamSnsPtr->raw[CapSensep_scanFreqIndex] = LO16(tmpRawData);
        }
        else
        {
            CapSensep_curRamSnsPtr->raw[CapSensep_scanFreqIndex] = LO16(tmpMaxCount);
        }

        #if (CapSensep_ENABLE == CapSensep_CSD_NOISE_METRIC_EN)
            /* Init bad Conversions number */
            CapSensep_badConversionsNum = CapSensep_BAD_CONVERSIONS_NUM;
        #endif /* (CapSensep_ENABLE == CapSensep_CSD_NOISE_METRIC_EN) */

        #if (CapSensep_ENABLE == CapSensep_CSD_SHIELD_EN)
            /* Open HCBV and HCBG switches */
            CY_SET_REG32(CapSensep_SW_SHIELD_SEL_PTR, CapSensep_SW_SHIELD_SEL_SW_HCBV_STATIC_OPEN |
                                                             CapSensep_SW_SHIELD_SEL_SW_HCBG_STATIC_OPEN);
        #endif /* (CapSensep_ENABLE == CapSensep_CSD_SHIELD_EN) */

    #else

        /* Read SlotResult from Raw Counter */
       CapSensep_curRamSnsPtr->raw[CapSensep_scanFreqIndex] = (uint16)CY_GET_REG32(CapSensep_COUNTER_PTR);

    #endif /* (CapSensep_ENABLE == CapSensep_CSDV2) */
}


/*******************************************************************************
* Function Name: CapSensep_SsCSDInitNextScan
****************************************************************************//**
*
* \brief
*   This function initializes the next sensor scan.
*
* \details
*   The function increments the sensor index, updates sense clock for matrix
*   or touchpad widgets only, sets up Compensation IDAC, enables the sensor and
*   scans it. When all the sensors are scanned it continues to set up the next widget
*   until all the widgets are scanned. The CSD block is disabled when all the widgets are
*   scanned.
*
*******************************************************************************/
static void CapSensep_SsCSDInitNextScan(void)
{
    /* Declare and initialize ptr to widget and sensor structures to appropriate address */
    #if (((CapSensep_ENABLE == CapSensep_CSD_IDAC_COMP_EN) || \
          (CapSensep_CSD_MATRIX_WIDGET_EN || CapSensep_CSD_TOUCHPAD_WIDGET_EN)) || \
          (((CapSensep_DISABLE == CapSensep_CSD_COMMON_SNS_CLK_EN) && \
          (CapSensep_ENABLE == CapSensep_CSDV2) && \
          (CapSensep_CSD_MATRIX_WIDGET_EN || CapSensep_CSD_TOUCHPAD_WIDGET_EN))))
        CapSensep_RAM_WD_BASE_STRUCT *ptrWdgt = (CapSensep_RAM_WD_BASE_STRUCT *)
                                                        CapSensep_dsFlash.wdgtArray[CapSensep_widgetIndex].ptr2WdgtRam;
    #endif

    /* Check if all the sensors are scanned in widget */
    if (((uint8)CapSensep_dsFlash.wdgtArray[(CapSensep_widgetIndex)].totalNumSns - 1u) > CapSensep_sensorIndex)
    {
        /* Increment sensor index to configure next sensor in widget */
        CapSensep_sensorIndex++;

        /* Update global pointer to CapSensep_RAM_SNS_STRUCT to current sensor  */
        CapSensep_curRamSnsPtr = (CapSensep_RAM_SNS_STRUCT *)
                                                  CapSensep_dsFlash.wdgtArray[CapSensep_widgetIndex].ptr2SnsRam
                                                  + CapSensep_sensorIndex;

        /* Configure clock divider to row or column */
        #if ((CapSensep_DISABLE == CapSensep_CSD_COMMON_SNS_CLK_EN) && \
             (CapSensep_CSD_MATRIX_WIDGET_EN || CapSensep_CSD_TOUCHPAD_WIDGET_EN))
            CapSensep_SsCSDConfigClock();

            #if (CapSensep_ENABLE == CapSensep_CSDV2)
                 /* Set up scanning resolution */
                CapSensep_SsCSDCalculateScanDuration(ptrWdgt);
            #endif
        #endif

        /* Setup Compensation IDAC for next sensor in widget */
        #if ((CapSensep_ENABLE == CapSensep_CSD_IDAC_COMP_EN) || \
             (CapSensep_CSD_MATRIX_WIDGET_EN || CapSensep_CSD_TOUCHPAD_WIDGET_EN))
            CapSensep_SsCSDSetUpIdacs(ptrWdgt);
        #endif /* ((CapSensep_ENABLE == CapSensep_CSD_IDAC_COMP_EN) || \
                   (CapSensep_CSD_MATRIX_WIDGET_EN || CapSensep_CSD_TOUCHPAD_WIDGET_EN)) */

        /* Enable sensor */
        CapSensep_SsCSDConnectSensorExt((uint32)CapSensep_widgetIndex, (uint32)CapSensep_sensorIndex);

        /* Proceed scanning */
        CapSensep_SsCSDStartSample();
    }
    /* Call scan next widget API if requested, if not, complete the scan  */
    else
    {
        CapSensep_sensorIndex = 0u;

        /* Current widget is totally scanned. Reset WIDGET BUSY flag */
        CapSensep_dsRam.status &= ~CapSensep_WDGT_SW_STS_BUSY;

        /* Check if all the widgets have been scanned */
        if (CapSensep_ENABLE == CapSensep_requestScanAllWidget)
        {
            /* Configure and begin scanning next widget */
            CapSensep_SsPostAllWidgetsScan();
        }
        else
        {
            #if (CapSensep_ENABLE == CapSensep_BLOCK_OFF_AFTER_SCAN_EN)
                #if (CapSensep_ENABLE == CapSensep_CSDV2)
                    /* Disable the CSD block */
                    CY_SET_REG32(CapSensep_CONFIG_PTR, CapSensep_configCsd);
                #else
                    /* Disable the CSD block. Connect Vref Buffer to AMUX bus */
                    #if ((CapSensep_CSH_PRECHARGE_IO_BUF == CapSensep_CSD_CSH_PRECHARGE_SRC) &&\
                         (CapSensep_ENABLE == CapSensep_CSD_SHIELD_TANK_EN))
                        CY_SET_REG32(CapSensep_CONFIG_PTR, CapSensep_configCsd | CapSensep_CMOD_PRECHARGE_CONFIG);
                    #else
                        CY_SET_REG32(CapSensep_CONFIG_PTR, CapSensep_configCsd | CapSensep_CTANK_PRECHARGE_CONFIG);
                    #endif
                #endif
            #endif

            /* All widgets are totally scanned. Reset BUSY flag */
            CapSensep_dsRam.status &= ~CapSensep_SW_STS_BUSY;

            /* Update scan Counter */
            CapSensep_dsRam.scanCounter++;
        }
    }
}

#if (CapSensep_ENABLE == CapSensep_MULTI_FREQ_SCAN_EN)
    /*******************************************************************************
    * Function Name: CapSensep_SsNextFrequencyScan
    ****************************************************************************//**
    *
    * \brief
    *   This function scans the sensor on the next frequency channel.
    *
    * \details
    *   The function increments the frequency channel, changes IMO and initializes
    *   the scanning process of the same sensor.
    *
    *******************************************************************************/
    static void CapSensep_SsNextFrequencyScan(void)
    {
        CapSensep_RAM_WD_BASE_STRUCT const *ptrWdgt = (CapSensep_RAM_WD_BASE_STRUCT *)
                                                        CapSensep_dsFlash.wdgtArray[CapSensep_widgetIndex].ptr2WdgtRam;

        CapSensep_scanFreqIndex++;

        /* Set Immunity */
        #if (CapSensep_MFS_IMO == CapSensep_MFS_METHOD)
            CapSensep_SsChangeImoFreq((uint32)CapSensep_scanFreqIndex);
        #else
            CapSensep_SsChangeClkFreq((uint32)CapSensep_scanFreqIndex);
        #endif /* (CapSensep_MFS_IMO == CapSensep_MFS_METHOD) */

        /* Update IDAC registers */
        CapSensep_SsCSDSetUpIdacs(ptrWdgt);

        /* Proceed scanning */
        CapSensep_SsCSDStartSample();
    }
#endif /* (CapSensep_ENABLE == CapSensep_MULTI_FREQ_SCAN_EN) */

#endif /* ((CapSensep_ENABLE == CapSensep_CSD_EN) || (CapSensep_ENABLE == CapSensep_CSD_CSX_EN)) */


/* [] END OF FILE */
