/*****************************************************************************
* File Name: main.c
*
* Version: 1.00
*
* Description: This example demonstrates how to tune a mutual capacitance (CSX)
*              based button using CapSense Tuner GUI.
*
* Related Document: CE228931_PSoC4_CapSense_CSX_Tuning.pdf
*
* Hardware Dependency: This example supports the following kits:
*                      CY8CKIT-149 PSoC® 4100S Plus Prototyping Kit,  
*                      CY8CKIT-041-41XX PSoC® 4100S CapSense Pioneer Kit, 
*                      CY8CKIT-041-40XX PSoC® 4 S-Series Pioneer Kit,  
*                      CY8CKIT-145-40XX PSoC® 4000S CapSense Prototyping Kit.
******************************************************************************
* Copyright (2020), Cypress Semiconductor Corporation.
******************************************************************************
* This software, including source code, documentation and related materials
* ("Software") is owned by Cypress Semiconductor Corporation (Cypress) and is
* protected by and subject to worldwide patent protection (United States and 
* foreign), United States copyright laws and international treaty provisions. 
* Cypress hereby grants to licensee a personal, non-exclusive, non-transferable
* license to copy, use, modify, create derivative works of, and compile the 
* Cypress source code and derivative works for the sole purpose of creating 
* custom software in support of licensee product, such licensee product to be
* used only in conjunction with Cypress's integrated circuit as specified in the
* applicable agreement. Any reproduction, modification, translation, compilation,
* or representation of this Software except as specified above is prohibited 
* without the express written permission of Cypress.
* 
* Disclaimer: THIS SOFTWARE IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND, 
* EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED 
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes to the Software without notice. 
* Cypress does not assume any liability arising out of the application or use
* of Software or any product or circuit described in the Software. Cypress does
* not authorize its products for use as critical components in any products 
* where a malfunction or failure may reasonably be expected to result in 
* significant injury or death ("ACTIVE Risk Product"). By including Cypress's 
* product in a ACTIVE Risk Product, the manufacturer of such system or application
* assumes all risk of such use and in doing so indemnifies Cypress against all
* liability. Use of this Software may be limited by and subject to the applicable
* Cypress software license agreement.
*****************************************************************************/

/*******************************************************************************
*   Included Headers
*******************************************************************************/

#include "project.h"

/*******************************************************************************
* Function Name: main()
********************************************************************************
* Summary:
* This function performs the following actions:
*
*     1. Starts the EZI2C component and exposes the CapSense_dsRam structure as 
*        the EZI2C buffer.
*     2. Starts the CapSense component.
*     3. Scans the CSX sensor in a continuous loop and processes the sensor  
*        status when scan is completed.
*     4. Calls CapSense_RunTuner() to establish synchronized communication with
*        Tuner application.
*
* Return:
* Non zero value indicates error 	
*
*******************************************************************************/
int main(void)
{
    /* Enable global interrupts. */
    CyGlobalIntEnable; 

    /* Start the EZI2C component to tune CapSense sensors */   
    EZI2C_Start();

    /* Set the CapSense data structure as the I2C buffer to be exposed to the 
     * master on primary slave address interface. Any I2C host tools such as 
     * the Tuner or the Bridge Control Panel can read this buffer but you can  
     * connect only one tool at a time.
     */
    EZI2C_EzI2CSetBuffer1(
                          sizeof(CapSense_dsRam),
                          sizeof(CapSense_dsRam),
                          (uint8 *) &CapSense_dsRam 
                         );

    /* Start the capsense block */
    CapSense_Start();
    
    //CapSense_CSDCalibrateWidget(CapSense_BUTTON0_WDGT_ID, 70u);
    CapSense_CSDCalibrateWidget(CapSense_PROXIMITY0_WDGT_ID, 70u);
    
    /* Scan all the enabled widgets */
    CapSense_ScanAllWidgets();
   
    for(;;)
    {
        /* Initiate a new scan only if the CapSense block is idle */ 
        if(CapSense_IsBusy() == CapSense_NOT_BUSY)
        {
            /* Performs full data processing of all enabled widgets */
            CapSense_ProcessAllWidgets();
            
            /* Sync with the Tuner application to fetch the latest CapSense  
             * parameters and update back the sensor data such as rawcount, 
             * baseline etc.
             */
            CapSense_RunTuner();
          
            /* Scan all the enabled widgets */
            CapSense_ScanAllWidgets();
            
            if(CapSense_IsWidgetActive(CapSense_PROXIMITY0_WDGT_ID) != 0){
            //if(CapSense_IsWidgetActive(CapSense_BUTTON0_WDGT_ID) != 0){
                LED_Write(1);
            }
            else{
                LED_Write(0);
            }
        } 
    }
}

/* [] END OF FILE */
